"""Experimental module to print basic information about a contract.

For now it gives a list of its methods (and try to resolve the name from their
signatures), and interesting properties inferred on them (onlyOwner? can we send
money to that method?...).
"""

import collections
import json
import lzma

from pakala import sm


# TODO: Don't hardcode it...
SIGNATURE_FILE = '../signatures.json.xz'


class HumanSummarizer:
    def __init__(self, symbolic_machine):
        self.signatures = json.load(lzma.open(SIGNATURE_FILE))
        self.signatures['0x00000000'] = 'fallback'
        for magic, signature in self.signatures.items():
            assert isinstance(signature, str)
            assert isinstance(magic, str)
            assert magic.startswith('0x'), magic
            assert len(magic) == 10, magic
            assert int(magic, 16) is not None  # Should not raise any exception

        self.sm = symbolic_machine

    def states_by_method(self):
        states = self.sm.outcomes + self.sm.partial_outcomes
        m = collections.defaultdict(list)
        for state in states:
            signature = 0
            solutions = state.solver.eval(state.env.calldata.read(0, 4), 2)
            assert solutions, "All states should be reachable"
            if len(solutions) == 1:
                signature = solutions[0]
            # If there are multiple solutions, signature stays null, which will be
            # the default method.
            m["{0:#010x}".format(signature)].append(state)
        return m


    def print_methods(self):
        for method, states in sorted(self.states_by_method().items()):
            signature = self.signatures.get(method, '')

            flags = set()
            all_not_payable = True
            all_only_payable = True
            for state in states:
                if state.calls:
                    flags.add('call')
                if state.suicide_to:
                    flags.add('suicide')

                for s in self.sm.partial_outcomes:
                    if state is s:
                        flags.add('errored')

                for s in self.sm.outcomes:
                    if state is s and state.is_interesting():
                        flags.add('interesting')

                if state.solver.satisfiable(extra_constraints=[state.env.value > 0]):
                    all_not_payable = False
                if state.solver.satisfiable(extra_constraints=[state.env.value == 0]):
                    all_only_payable = False

                # TODO: actually read the storage...
                read_constraints = [v == 1 for v in state.storage_read.values()]
                try:
                    callers = state.solver.eval(state.env.caller[159:0], 2,
                                                extra_constraints=read_constraints)
                    if len(callers) == 1:
                        flags.add('onlyOwner')
                except Exception:
                    pass

                state.solver.downsize()

            if all_not_payable:
                flags.add("notPayable")
            if all_only_payable:
                flags.add("onlyPayable")

            print('%s %s %s' % (method, signature.ljust(40), ' '.join(sorted(flags))))



"""
    pakala: EVM symbolic execution tool and vulnerability scanner.
    Copyright (C) 2018 Korantin Auguste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
import collections
import logging
import functools
import datetime
import time
import itertools
import pprint

import claripy

from pakala import analyzer
from pakala import env
from pakala import utils
from pakala.state import State

logger = logging.getLogger(__name__)


DEBUG_MARK_PATH = []


def is_function(state, function):
    return state.solver.satisfiable([state.env.calldata.read(0, 4) == function])


def with_new_env(state):
    assert state.solver.satisfiable()
    old_env = state.env
    new_env = old_env.clean_copy()
    state = state.copy()
    state.env = new_env
    state.replace(functools.partial(env.replace, old_env, new_env))

    for read_k, read_v in state.storage_read.items():
        new_v = claripy.BVS('storage[%s]' % read_k, 256)
        state.replace(lambda ast: ast.replace(read_v, new_v))

    assert state.solver.satisfiable()
    return state


class RecursiveAnalyzer(analyzer.BaseAnalyzer):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Each element in the queue is a couple:
        # 0) State, a state made of the successive applications of all the state
        #    of the path, except the last one.
        # 1) list, path of states we applied.
        self.path_queue = collections.deque()

        # The dict in the queue before
        self.state_done = set()

        self.reference_states = []

    @property
    def address(self):
        return self.reference_states[0][0].env.address

    @property
    def caller(self):
        return self.reference_states[0][0].env.caller

    def _search_path(self, composite_state, path):
        logger.debug("Search path: %s", path)
        logger.debug("Composite state: %s", composite_state)

        # If we already encountered the same composite state with some other
        # path...
        if hash(composite_state) in self.state_done:
            return
        self.state_done.add(hash(composite_state))

        logger.debug("Check for bugs in composite state...")
        if self.check_state(composite_state, path=path):
            return path

        # If we kill the contract, we can't make any more call!
        if path[-1].suicide_to is not None:
            return

        # We have to downsize the used solver to free memory in z3, otherwise
        # they will collectively end up eating all the memory.
        composite_state.solver.downsize()

        # For each reference state, find the right one (with an unused env)
        # and add it to the queue.
        for reference_states in self.reference_states:
            for reference_state in reference_states:
                if all(s is not reference_state for s in path):
                    self.path_queue.append(
                        (composite_state.copy(), path + [reference_state]))
                    break

    def _append_state(self, composite_state, state):
        # May fail because pprint compare claripy symbols. So only if needed.
        if logger.isEnabledFor(logging.DEBUG):
            logger.debug("_append_state: appending state %s\nto composite state %s",
                        pprint.pformat(state.as_dict()),
                        pprint.pformat(composite_state.as_dict()))

        assert composite_state.suicide_to is None

        composite_state.solver = composite_state.solver.combine([state.solver])

        if not composite_state.solver.satisfiable():
            return []
        assert state.solver.satisfiable()

        for call in state.calls:
            composite_state.calls.append(call)

        if state.suicide_to is not None:
            composite_state.suicide_to = state.suicide_to

        # Resolve read/write

        def apply_read(r_key, r_val, composite_state):
            """Apply a read operation with (key, value) to the state."""
            composite_states_next = []

            # Here we consider the cases where it's possible to read something
            # we previously wrote to.
            not_overwritten_c = []
            for w_key, w_val in composite_state.storage_written.items():
                read_written = [r_key == w_key, r_val == w_val]
                not_overwritten_c.append(r_key != w_key)

                if composite_state.solver.satisfiable(extra_constraints=read_written):
                    cs = composite_state.copy()
                    cs.solver.add(read_written)
                    composite_states_next.append(cs)
                    logger.debug("Found key read %s, corresponding to key written %s", r_key, w_key)

            # Is it not something we previously wrote to?
            composite_state.solver.add(not_overwritten_c)
            composite_state.solver.add(
                state.storage_read[r_key] == self._read_storage(state, r_key))
            if composite_state.solver.satisfiable():
                composite_states_next.append(composite_state)

            return composite_states_next

        composite_states = [composite_state]
        for r_key, r_val in state.storage_read.items():
            composite_states = list(itertools.chain.from_iterable(
                apply_read(r_key, r_val, composite_state)
                for composite_state in composite_states))

        for composite_state in composite_states:
            # Delete any storage_written at the same key in the composite
            # state.
            for c_key in list(composite_state.storage_written.keys()):
                for key in state.storage_written.keys():
                    if not composite_state.solver.satisfiable(extra_constraints=[key != c_key]):
                        del composite_state.storage_written[c_key]
                        break

            for key, val in state.storage_written.items():
                composite_state.storage_written[key] = val

        # May fail because pprint compare claripy symbols. So only if needed.
        if logger.isEnabledFor(logging.DEBUG):
            logger.debug("_append_state: found states: %s",
                         pprint.pformat([composite_state.as_dict() for composite_state in composite_states]))

        return composite_states

    def check_states(self, states, timeout, max_depth):
        states = [state for state in states if state.is_interesting()]
        if not states:
            return

        # Each state must have its own independent environment
        assert not self.reference_states

        # For each state, a list of equivalent state, but each in a different
        # env so that they can be stacked together.
        self.reference_states = []
        for state in states:
            self.reference_states.append(
                [with_new_env(state) for _ in range(max_depth)])
            # Add it to the paths to explore
            self.path_queue.append(
                (State(), [self.reference_states[-1][0]]))

        # Recursive exploration
        last_path_len = 1
        time_start = time.process_time()
        while self.path_queue:
            initial_composite_state, path = self.path_queue.popleft()

            if len(path) > last_path_len:
                logger.log(utils.INFO_INTERACTIVE,
                           "Now scanning paths of length %i.", len(path))
                last_path_len = len(path)
            if len(path) > max_depth:
                logger.debug("Over the max allowed depth, stopping.")
                return

            if (DEBUG_MARK_PATH and
                all(is_function(s, f) for s, f in zip(path, DEBUG_MARK_PATH))):
                logger.warning('DEBUG_MARK_PATH len %i', len(path))
                logger.warning('path: %s', path)
                breakpoint()

            new_composite_states = self._append_state(
                    initial_composite_state, path[-1])

            for composite_state in new_composite_states:
                if self._search_path(composite_state, path) is not None:
                    return composite_state, path

            if timeout and time.process_time() - time_start > timeout:
                logger.debug("Timeout at depth %i, stopping.", len(path))
                return



import collections
import logging
import numbers

import claripy

from pakala import env
from pakala import utils

from web3 import Web3


logger = logging.getLogger(__name__)


class BaseAnalyzer(object):
    """Base class for an Analyzer.

    Child classes need to define `caller` and `address`.
    """

    def __init__(self, max_wei_to_send, min_wei_to_receive, block='latest'):
        self.web3 = Web3()
        self.web3.eth.defaultBlock = block
        self.max_wei_to_send = max_wei_to_send
        self.min_wei_to_receive = min_wei_to_receive
        self.storage_cache = {}

    def _read_storage(self, state, key):
        # TODO: We do an approximation here: if it cannot be computed or it can
        # be multiple things, we assume the initial storage is 0...
        # Instead we could use a cascade of claripy.If(key, value, claripy.If(...
        # reflecting the actual storage (if there are not too many keys in storage.
        logger.debug("Reading storage %r" % key)
        try:
            keys = state.solver.eval(key, 2)
            if len(keys) > 1:
                logger.info("Multiple values possible for key %r", key)
                return utils.bvv(0)
            assert len(keys) == 1
            key = keys[0]
            assert isinstance(key, numbers.Number)
        except claripy.errors.UnsatError as e:
            # Should not be too bad, because for the same key we will reuse the
            # same cache.
            logger.debug(
                "Encountered an exception when resolving key %r: %r", key, e)
            return utils.bvv(0)

        if key in self.storage_cache:
            value = self.storage_cache[key]
        else:
            hex_addr = self.web3.toChecksumAddress(
                    utils.number_to_address(
                        utils.bvv_to_number(self.address)))
            value = self.web3.toInt(self.web3.eth.getStorageAt(hex_addr, key))
            self.storage_cache[key] = value

        return utils.bvv(value)

    def check_state(self, state, path=None):
        """Check a reachable state for bugs"""
        logger.debug("Check state: %s", state)
        logger.debug("Constraints: %s", state.solver.constraints)

        read_constraints = []
        extra_constraints = []  # From the environment (block number, whatever)

        if path is None:
            path = [state]
            # Static read were we never wrote, but we know the key is not symbolic.
            # So we go and fetch it.
            for key, value in state.storage_read.items():
                constraint = state.storage_read[key] == self._read_storage(state, key)
                read_constraints.append(constraint)
                logger.debug("Add constraint: %s", constraint)

        for s in path:
            extra_constraints += s.env.extra_constraints()
            extra_constraints += [
                s.env.caller == utils.DEFAULT_CALLER,
                s.env.origin == utils.DEFAULT_CALLER]

        # Calls
        total_sent = sum(s.env.value for s in path)
        sent_constraints = [s.env.value < self.max_wei_to_send for s in path]
        total_received_by_me = utils.bvv(0)
        total_received_by_others = utils.bvv(0)

        for call in state.calls:
            value, to, gas = call[-3:]  # pylint: disable=unused-variable,invalid-name
            if state.solver.satisfiable(
                    extra_constraints=[to[159:0] == self.caller[159:0]]):
                state.solver.add(to[159:0] == self.caller[159:0])
                total_received_by_me += value
            else:
                total_received_by_others += value

        final_balance = (
                path[0].env.balance + total_sent
                - total_received_by_me - total_received_by_others)

        # Suicide
        if state.suicide_to is not None:
            constraints = extra_constraints + read_constraints + [
                final_balance >= self.min_wei_to_receive,
                state.suicide_to[159:0] == self.caller[159:0]]
            logger.debug("Check for suicide bug with constraints %s", constraints)
            if state.solver.satisfiable(extra_constraints=constraints):
                logger.info("Found suicide bug.")
                return True

        if total_received_by_me is utils.bvv(0):
            return False

        logger.debug("Found calls back to caller: %s", total_received_by_me)

        constraints = sent_constraints + extra_constraints + read_constraints + [
            final_balance >= 0,
            total_received_by_me > total_sent,  # I get more than what I sent?
            total_received_by_me > self.min_wei_to_receive]

        logger.debug("Extra constraints: %r", constraints)

        if state.solver.satisfiable(extra_constraints=constraints):
            logger.info("Found call bug.")
            return True

        return False


class Analyzer(BaseAnalyzer):
    """Simple Analyzer class, where caller and address are given explicitly."""

    def __init__(self, address, caller, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.caller = caller
        self.address = address


class FakeStorage(dict):
    """For testing, you can override the storage_cache with an instance of this
    class. This allow you to simulate storage for a contract, and prevent the
    code to try to fetch it from web3. Instead it will crash if it tries to
    access something that you didn't specify."""

    def __contains__(self, key):
        if not super().__contains__(key):
            raise KeyError("The analyzer is trying to access a FakeStorage"
                           " key that we didn't specify: '%s'." % key)
        return True


class EmptyStorage(object):
    """For testing, you can override the storage_cache with an instance of this
    class, which will simulate a completely empty storage."""

    def __contains__(self, key):
        return True

    def __getitem__(self, key):
        return 0




import claripy
import logging

from pakala import memory
from pakala import utils

logger = logging.getLogger(__name__)


class State(object):
    """Represents a state during the execution of a contract.
    It also contains the interactions with the world.
    """
    def __init__(self, env=None):
        self.env = env
        self.pc = 0 # pylint:disable=invalid-name
        self.stack = []

        # TODO: explain
        self.score = 0

        self.memory = memory.Memory()

        # That's an override to the storage in the blockchain.
        # It's the storage that has been written at the end of the execution of
        # the contract.
        self.storage_written = {}

        # Storage read while executing the contract.
        self.storage_read = {}

        self.calls = []
        self.suicide_to = None

        self.solver = utils.get_solver()

    def __repr__(self):
        return ("State(suicide_to=%s, calls=%s, storage_written=%s, "
                "storage_read=%s, env=%s, solver=%s)") % (
                    self.suicide_to, self.calls, self.storage_written,
                    self.storage_read, self.env, self.solver)

    def as_dict(self):
        return {'suicide_to': self.suicide_to,
                'calls': self.calls,
                'storage_written': self.storage_written,
                'storage_read': self.storage_read,
                'env': None if self.env is None else self.env.as_dict(),
                'solver': self.solver.as_dict()}

    def clean(self):
        """Clean the state, when it won't be executed anymore and we are only
        interested by the calls, suicides..."""
        self.stack = []
        self.memory = memory.Memory()
        self.solver.downsize()

    def replace(self, r):
        """Call r() repeatedly, with every single AST that's present in the
        state:
            - in storage, read and written
            - calls
            - in suicide data
            - in solver constraints
        r() can replace symbols in the AST by other symbols. Generally, r() is
        derived from Env.replace(), to substitute an environment with another.
        """
        logger.debug("State.replace %s", r)
        self.storage_written = {
            r(k): r(v)
            for k, v in self.storage_written.items()}
        self.storage_read = {
            r(k): r(v)
            for k, v in self.storage_read.items()}
        self.calls = [[r(i) for i in call] for call in self.calls]
        self.suicide_to = None if self.suicide_to is None else r(self.suicide_to)

        # TODO: Do something cleaner! This work only with our custom solver mixin.
        self.solver.replace(r)
        #constraints = [r(i) for i in self.solver.constraints]
        #self.solver = utils.get_solver()
        #for c in constraints:
        #    self.solver.add(c)

    def __hash__(self):
        l = [hash(self.env), hash(self.pc), hash(self.memory),
             hash(self.suicide_to)]
        for i in self.stack:
            l.append(hash(i))
        for call in self.calls:
            for arg in call:
                l.append(hash(arg))
        # The following is because the ordering shouldn't matter:
        x = 0
        for k, v in self.storage_written.items():
            x ^= hash((k, v))
        l.append(x)
        for k, v in self.storage_read.items():
            x ^= hash((k, v))
        l.append(x)
        for constraint in self.solver.constraints:
            x ^= hash(constraint)
        l.append(x)
        return hash(tuple(l))

    def stack_push(self, x):
        if len(self.stack) >= 1024:
            raise utils.CodeError("Stack overflow")
        self.stack.append(x)

    def stack_pop(self):
        if not self.stack:
            raise utils.CodeError("Stack underflow")
        return self.stack.pop()

    def is_interesting(self):
        return bool(self.storage_written
                    or self.calls
                    or self.suicide_to is not None)

    def copy(self):
        """Make a shallow copy of the current environment. Needs to be fast."""
        new_state = State(self.env)
        new_state.pc = self.pc
        new_state.stack = self.stack[:]
        new_state.memory = self.memory.copy()
        new_state.storage_written = self.storage_written.copy()
        new_state.storage_read = self.storage_read.copy()
        new_state.solver = self.solver.branch()
        new_state.calls = self.calls[:]
        new_state.suicide_to = self.suicide_to
        new_state.score = self.score
        return new_state

    # TODO(palkeo): Get rid of that. Needed because of heapq in sm.py...
    def __eq__(self, other):
      return 1
    def __ne__(self, other):
      return 0
    __lt__ = __ne__
    __gt__ = __ne__
    __ge__ = __eq__
    __le__ = __eq__




import logging

import claripy

from pakala import utils

logger = logging.getLogger(__name__)

# Maximum size the memory is allowed to have.
MEMORY_SIZE = 4096


def _slice(v, start, end):
    start = v.size() - start*8 - 1
    end = 0 if end is None else (v.size() - end*8)
    assert end >= 0 and start >= 0 and end <= start
    return v[start:end]


class Memory(object):
    """Base class for memory. Uninitialized memory is zero initially."""
    def __init__(self):
        self._mem = {}

    def __str__(self):
        return str(self._mem)

    def __hash__(self):
        return hash(tuple(
            hash(k) ^ hash(v) for k, v in self._mem.items()))

    def _default(self, addr, size):
        return claripy.BVV(0, size*8)

    def read(self, addr, size):
        if size < 1:
            raise utils.CodeError("size < 1 in Memory.read")
        if addr + size >= MEMORY_SIZE:
            raise utils.CodeError("overflow in Memory.read")
        logger.debug("%s.read(%i, %i)" % (self.__class__.__name__, addr, size))

        for iaddr, ivalue in self._mem.items():
            isize = ivalue.size() // 8
            raddr = iaddr - addr
            rend = iaddr + isize - addr
            # completely overlaps (or equals)
            if raddr <= 0 and rend >= size:
                return _slice(ivalue, -raddr, -raddr+size)
            # completely inside (strictly)
            elif raddr > 0 and rend < size:
                return self.read(addr, raddr).concat(self.read(addr + raddr, size - raddr))
            # end inside
            elif 0 < rend < size:
                return _slice(ivalue, isize - rend, None).concat(self.read(addr + rend, size - rend))
            # start inside
            elif 0 < raddr < size:
                return self.read(addr, raddr).concat(_slice(ivalue, 0, size - raddr))

        assert addr not in self._mem
        self._mem[addr] = self._default(addr, size)
        return self._mem[addr]

    def write(self, addr, size, value):
        if size < 1:
            raise utils.CodeError("size < 1 in Memory.write")
        if addr + size >= MEMORY_SIZE:
            raise utils.CodeError("overflow in Memory.write")
        if value.size() // 8 != size:
            raise utils.InterpreterError("BVV size doesn't match size in Memory.write")

        logger.debug("%s.write(%i, %i, %r)" % (self.__class__.__name__, addr, size, value))

        for iaddr, ivalue in list(self._mem.items()):
            isize = ivalue.size() // 8
            raddr = iaddr - addr
            rend = iaddr + isize - addr
            # equal
            if raddr == 0 and rend == size:
                break
            # completely overlaps
            elif raddr <= 0 and rend >= size:
                if raddr < 0:
                    self._mem[iaddr] = _slice(ivalue, 0, -raddr)
                self._mem[addr + size] = _slice(ivalue, -raddr+size, None)
            # completely inside (not strictly)
            elif raddr >= 0 and rend <= size:
                del self._mem[iaddr]
            # end inside
            elif 0 < rend < size:
                self._mem[iaddr] = _slice(ivalue, 0, -raddr)
            # start inside
            elif 0 < raddr < size:
                del self._mem[iaddr]
                self._mem[addr + size] = _slice(ivalue, size - raddr, None)

        self._mem[addr] = value

    def copy_from(self, other, start_self, start_other, size):
        if size < 1:
            raise utils.CodeError("size < 1 in Memory.copy_from")
        if start_self + size >= MEMORY_SIZE or start_other + size >= MEMORY_SIZE:
            raise utils.CodeError("overflow in Memory.copy_from")
        self.write(start_self, size, CalldataMemoryView(other, start_other, size))

    def size(self):
        if not self._mem:
            return 0
        max_addr = max(self._mem.keys())
        return max_addr + self._mem[max_addr].size() // 8

    def copy(self):
        """Not to be confused with copy_from. This is to copy the object, not
        do any memory operation.
        """
        new_memory = self.__class__()
        new_memory._mem = self._mem.copy()
        return new_memory


class CalldataMemory(Memory):
    """Same as Memory, except that uninitialized memory is set to a BVS."""
    def _default(self, addr, size):
        return claripy.BVS('calldata[%i]' % addr, size*8)

    def write(self, *args, **kwargs):
        assert False, "CalldataMemory is read-only."

    def copy_from(self, *args, **kwargs):
        assert False, "CalldataMemory is read-only."


class CalldataMemoryView(object):
    """Element to be put in memory like a Claripy BV, but is a view to a part
    of another memory."""
    def __init__(self, mem, addr, size):
        self._mem = mem
        self._addr = addr
        self._size = size

    def __hash__(self):
      # TODO: The hash may change even if the part in the view don't...
      return hash(self._mem)

    def size(self):
        return self._size * 8

    def __getitem__(self, item):
        # If I read the whole view, the thing will crash.
        assert isinstance(item, slice)
        assert (item.start + 1) % 8 == 0 and item.stop % 8 == 0 and not item.step
        start = self._size - (item.start + 1) // 8
        stop = self._size - item.stop // 8
        size = stop - start
        assert size <= self._size and start >= 0
        return self._mem.read(self._addr + start, size)



import claripy
import unittest
import logging

from pakala.analyzer import Analyzer, FakeStorage
from pakala.env import Env
from pakala.state import State
from pakala import utils

from web3 import Web3

logging.basicConfig(level=logging.DEBUG)


class TestCheckState(unittest.TestCase):
    def setUp(self):
        self.env = Env(b'', caller=utils.DEFAULT_CALLER,
                            address=utils.DEFAULT_ADDRESS)

        self.state = State(self.env)
        self.analyzer = Analyzer(address=self.env.address,
                                 caller=self.env.caller,
                                 max_wei_to_send=Web3.toWei(10, 'ether'),
                                 min_wei_to_receive=Web3.toWei(1, 'milliether'))

    def check_state(self, state):
        return self.analyzer.check_state(state)

    def get_call(self, value, to=None):
        if to is None:
            to = self.env.caller
        return [utils.bvv(0), utils.bvv(0), utils.bvv(0), utils.bvv(0),
                value, to, utils.bvv(0)]

    def test_nothing(self):
        self.assertFalse(self.check_state(
            self.state))

    def test_suicide(self):
        self.state.suicide_to = self.env.caller
        self.assertTrue(self.check_state(
            self.state))

    def test_send_back(self):
        self.state.calls.append(self.get_call(self.env.value))
        self.assertFalse(self.check_state(
            self.state))

    def test_send_back_more(self):
        self.state.calls.append(self.get_call(
            self.env.value + Web3.toWei(1, 'ether')))
        self.assertTrue(self.check_state(
            self.state))

    def test_send_back_if_impossible_block(self):
        self.state.calls.append(self.get_call(
            claripy.If(self.env.block_number > 100000000000,
                       self.env.value + Web3.toWei(1, 'ether'), 0)))
        self.assertFalse(self.check_state(
            self.state))

    def test_send_back_if_possible_block(self):
        self.state.calls.append(self.get_call(
            claripy.If(self.env.block_number < 100000000000,
                       self.env.value + Web3.toWei(1, 'ether'), 0)))
        self.assertTrue(self.check_state(
            self.state))

    def test_send_back_nothing(self):
        self.state.calls.append(self.get_call(
            utils.bvv(0)))
        self.assertFalse(self.check_state(
            self.state))

    def test_send_back_twice(self):
        self.state.calls.append(self.get_call(
            self.env.value))
        self.state.calls.append(self.get_call(
            self.env.value / 10))
        self.assertTrue(self.check_state(
            self.state))

    def test_send_back_fixed_amount(self):
        self.state.calls.append(self.get_call(
            Web3.toWei(1, 'ether')))
        self.assertTrue(self.check_state(
            self.state))

    def test_send_back_to_someone_else(self):
        self.state.calls.append(self.get_call(
            Web3.toWei(1, 'ether'), to=self.env.caller + 1))
        self.assertFalse(self.check_state(
            self.state))

    def test_send_all(self):
        self.state.calls.append(self.get_call(
            self.env.balance))
        self.assertTrue(self.check_state(
            self.state))

    # TODO: Fix it!
    @unittest.skip("Known issue: we are sending back env.balance, that doesn't contain env.value, and it should!")
    def test_send_all_and_suicide(self):
        self.state.calls.append(self.get_call(
            self.env.balance, to=self.env.caller + 1))
        self.state.suicide_to = self.env.caller
        self.assertFalse(self.check_state(
            self.state))

    def test_read_concrete(self):
        self.analyzer.storage_cache = FakeStorage({0: 0xBAD1DEA})

        self.state.storage_read[utils.bvv(0)] = claripy.BVS('storage[0]', 256)
        self.state.suicide_to = self.state.storage_read[utils.bvv(0)]
        self.assertFalse(self.check_state(self.state))

        self.state.calls.append(self.get_call(
            Web3.toWei(1, 'ether') * self.state.storage_read[utils.bvv(0)]))
        self.assertTrue(self.check_state(self.state))

    def test_fakestorage_raises(self):
        """If the code accesses a FakeStorage key that we didn't specify,
           it should crash (only meant for testing)."""
        self.analyzer.storage_cache = FakeStorage({42: 0xBAD1DEA})
        self.state.storage_read[utils.bvv(0)] = claripy.BVS('storage[0]', 256)
        with self.assertRaises(KeyError):
            self.check_state(self.state)



import unittest
import functools

import claripy

from pakala.claripy_sha3 import Sha3
from pakala.utils import get_solver
from pakala import env
from pakala.state import State


class TestSha3Support(unittest.TestCase):
    def test_sha3_equality(self):
        a = claripy.BVV(1, 256)
        b = claripy.BVV(2, 256)
        self.assertEqual(Sha3(a), Sha3(claripy.BVV(1, 256)))
        self.assertNotEqual(Sha3(a), Sha3(b))

    def test_solver_basic(self):
        s = get_solver()
        in1 = claripy.BVS('in1', 256)
        in2 = claripy.BVS('in2', 256)

        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1) == Sha3(in2)]))
        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1) != Sha3(in2)]))
        # These next two always hold anyway.
        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 != Sha3(in2)]))
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 == Sha3(in2)]))

        s.add(in1 == in2)
        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1) == Sha3(in2)]))
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(in1) != Sha3(in2)]))
        # These next two always hold anyway.
        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 != Sha3(in2)]))
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 == Sha3(in2)]))

        s = get_solver()
        s.add(in1 != in2)
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(in1) == Sha3(in2)]))
        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1) != Sha3(in2)]))
        # These next two always hold anyway.
        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 != Sha3(in2)]))
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 == Sha3(in2)]))

    def test_solver_arithmetics(self):
        s = get_solver()
        in1 = claripy.BVS('in1', 256)
        in2 = claripy.BVS('in2', 256)

        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 == Sha3(in2) + 1]))
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 == Sha3(in2) + 2]))
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(in1) + 1 == Sha3(in2) + 2]))

        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1 + 1) == Sha3(in2 + 1)]))
        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(in1 + 1) == Sha3(in2 - 1)]))
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(in1 + 1) + 42 == Sha3(in2 - 1)]))

    def test_solver_one_var(self):
        s = get_solver()
        in1 = claripy.BVS('in1', 256)

        self.assertFalse(s.satisfiable(extra_constraints=[Sha3(in1) == 42]))
        self.assertFalse(s.satisfiable(extra_constraints=[Sha3(in1) == 0]))
        self.assertTrue(s.satisfiable(extra_constraints=[Sha3(in1) == Sha3(42)]))
        self.assertTrue(s.satisfiable(extra_constraints=[Sha3(in1) == Sha3(0)]))
        self.assertTrue(s.satisfiable(extra_constraints=[Sha3(in1 + 1) + 2 == Sha3(0) + 2]))

    def test_solver_recursive(self):
        s = get_solver()
        in1 = claripy.BVS('in1', 256)
        in2 = claripy.BVS('in2', 256)

        self.assertFalse(s.satisfiable(extra_constraints=[Sha3(Sha3(in1)) == 0]))
        self.assertFalse(s.satisfiable(extra_constraints=[Sha3(Sha3(in1)) == Sha3(0)]))
        self.assertTrue(s.satisfiable(extra_constraints=[Sha3(Sha3(in1)) == Sha3(Sha3(0))]))

        s.add(Sha3(in1) == Sha3(in2))
        self.assertTrue(s.satisfiable())

        self.assertTrue(s.satisfiable(
            extra_constraints=[Sha3(Sha3(in1) + 1) == Sha3(Sha3(in2) + 1)]))

        s.add(Sha3(Sha3(in1) + 1) == Sha3(Sha3(in2) + 1))
        self.assertTrue(s.satisfiable())

        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(Sha3(in1) + 2) == Sha3(Sha3(in2) + 1)]))
        self.assertFalse(s.satisfiable(
            extra_constraints=[Sha3(Sha3(in1)) + 1 == Sha3(Sha3(in2) + 1)]))

        s.add(Sha3(Sha3(in1)) + 3 == Sha3(Sha3(in2)) + 1)
        self.assertFalse(s.satisfiable())

        s_copy = s.branch()
        self.assertFalse(s.satisfiable())

    def test_solver_copy(self):
        s = get_solver()
        in1 = claripy.BVS('in1', 256)
        s.add(Sha3(in1) == 0)
        self.assertFalse(s.satisfiable())
        s2 = s.branch()
        self.assertFalse(s2.satisfiable())

    def test_env_replace_merge(self):
        old_env = env.Env(b'')
        new_env = old_env.clean_copy()

        old_state = State(old_env)
        old_state.solver.add(Sha3(old_env.caller) == old_env.value)

        self.assertTrue(old_state.solver.satisfiable())
        self.assertFalse(old_state.solver.satisfiable(extra_constraints=[old_env.value == 5]))

        new_state = old_state.copy()
        new_state.replace(functools.partial(env.replace, old_env, new_env))

        self.assertTrue(new_state.solver.satisfiable())
        self.assertFalse(new_state.solver.satisfiable(extra_constraints=[new_env.value == 5]))
        self.assertTrue(new_state.solver.satisfiable(extra_constraints=[old_env.value == 5]))

        new_state.solver.add(old_env.value == new_env.value)
        self.assertTrue(new_state.solver.satisfiable())
        self.assertFalse(new_state.solver.satisfiable(extra_constraints=[new_env.value == 5]))
        self.assertFalse(new_state.solver.satisfiable(extra_constraints=[old_env.value == 5]))

        old_state.solver = old_state.solver.combine([new_state.solver])
        self.assertTrue(new_state.solver.satisfiable())
        self.assertEqual(len(old_state.solver.constraints), 3)
        self.assertEqual(len(old_state.solver.hashes), 2)


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest import mock
import random
import numbers

import claripy

from pakala.env import Env
from pakala import sm
from pakala import utils

class TestSymbolicMachine(unittest.TestCase):
    """Basic tests for the members of the symbolic machine."""

    def setUp(self):
        code = "6003600302600f56601b60006000a15b6101a560006000a160019003801515600f57600660006000a1"
        code = utils.hex_to_bytes(code)
        env = Env(code)

        self.sm = sm.SymbolicMachine(env)

    def test_add_branch(self):
        state = self.sm.branch_queue[0][1]
        state.pc += 1

        self.sm.add_branch(state)
        self.assertEqual(len(self.sm.branch_queue), 2)

        # Add duplicate branch
        self.sm.add_branch(state)
        self.assertEqual(len(self.sm.branch_queue), 2)
        self.assertEqual(len(self.sm.code_errors), 1)

        # Add unsatisfiable state
        state.solver.add(claripy.BVS('lol', 8) == 5)
        self.sm.add_branch(state)
        self.assertEqual(len(self.sm.branch_queue), 3)
        state.solver.add(claripy.BVV(1, 8) == claripy.BVV(0, 8))
        self.sm.add_branch(state)
        self.assertEqual(len(self.sm.branch_queue), 3)
        self.assertEqual(len(self.sm.code_errors), 2)

    def test_add_for_fuzzing(self):
        state = self.sm.branch_queue[0][1]
        variable = claripy.BVS('test', 8)
        state.solver.add(variable < 50)
        self.sm.add_for_fuzzing(state, variable, [min, max, None, None, None, None, 42, 1337])
        self.assertTrue(any(s.solver.eval(variable, 2) == (0,)
                        for _, s in self.sm.branch_queue))
        self.assertTrue(any(s.solver.eval(variable, 2) == (49,)
                        for _, s in self.sm.branch_queue))
        self.assertTrue(any(s.solver.eval(variable, 2) == (42,)
                        for _, s in self.sm.branch_queue))
        self.assertGreater(len(self.sm.branch_queue), 3)

    def test_execute(self):
        self.sm.execute(timeout_sec=10)

    def test_get_coverage(self):
        # Just test that it seems to work
        self.assertEqual(self.sm.get_coverage(), 0)

class TestInstructions(unittest.TestCase):
    def assertBEqual(self, a, b):
        self.assertTrue((a == b).is_true(), msg="%s != %s" % (a, b))

    @mock.patch.object(utils, 'disassemble', lambda x: x)
    def run_code(self, code, env={}):
        self.sm = sm.SymbolicMachine(Env(code, **env))
        self.state = self.sm.branch_queue[0][1]
        return self.sm.exec_branch(self.state)

    def assert_stack(self, stack):
        self.assertEqual(len(stack), len(self.state.stack))
        for reference, observed in zip(stack, self.state.stack):
            self.assertBEqual(reference, observed)

    def test_jumpdest(self):
        self.assertTrue(self.run_code(['JUMPDEST']))

    def test_add(self):
        self.run_code(['PUSH1', 1, 'PUSH1', 2, 'ADD'])
        self.assert_stack([3])
        self.run_code(['PUSH1', 1, 'PUSH1', 2**256 - 1, 'ADD'])
        self.assert_stack([0])

    def test_sub(self):
        self.run_code(['PUSH1', 2, 'PUSH1', 1, 'SUB'])
        self.assert_stack([claripy.BVV(2**256 - 1, 256)])

    def test_mul(self):
        self.run_code(['PUSH1', 2, 'PUSH1', 4, 'MUL'])
        self.assert_stack([8])

    def test_div(self):
        self.run_code(['PUSH1', 2, 'PUSH1', 8, 'DIV'])
        self.assert_stack([4])
        self.run_code(['PUSH1', 3, 'PUSH1', 8, 'DIV'])
        self.assert_stack([2])
        self.run_code(['PUSH1', 10, 'PUSH1', 8, 'DIV'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 2**256 - 1, 'PUSH1', 1, 'DIV'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0, 'PUSH1', 1, 'DIV'])
        self.assert_stack([0])

    def test_sdiv(self):
        self.run_code(['PUSH1', 2**256 - 1, 'PUSH1', 1, 'SDIV'])
        self.assert_stack([2**256 - 1])
        self.run_code(['PUSH1', 0, 'PUSH1', 1, 'SMOD'])
        self.assert_stack([0])

    def test_mod(self):
        self.run_code(['PUSH1', 3, 'PUSH1', 7, 'MOD'])
        self.assert_stack([1])
        self.run_code(['PUSH1', 3, 'PUSH1', 2**256 - 1, 'MOD'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0, 'PUSH1', 1, 'MOD'])
        self.assert_stack([0])

    def test_smod(self):
        self.run_code(['PUSH1', 3, 'PUSH1', 2**256 - 1, 'SMOD'])
        self.assert_stack([2**256 - 1])
        self.run_code(['PUSH1', 3, 'PUSH1', 2**256 - 4, 'SMOD'])
        self.assert_stack([2**256 - 1])
        self.run_code(['PUSH1', 3, 'PUSH1', 4, 'SMOD'])
        self.assert_stack([1])
        self.run_code(['PUSH1', 0, 'PUSH1', 1, 'SMOD'])
        self.assert_stack([0])

    def test_addmod(self):
        self.run_code(['PUSH1', 2, 'PUSH1', 1, 'PUSH1', 1, 'ADDMOD'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 2, 'PUSH1', 2, 'PUSH1', 1, 'ADDMOD'])
        self.assert_stack([1])
        self.run_code(['PUSH1', 2, 'PUSH1', 2, 'PUSH1', 2, 'ADDMOD'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0, 'PUSH1', 2, 'PUSH1', 1, 'ADDMOD'])
        self.assert_stack([0])

    def test_mulmod(self):
        self.run_code(['PUSH1', 2, 'PUSH1', 1, 'PUSH1', 1, 'MULMOD'])
        self.assert_stack([1])
        self.run_code(['PUSH1', 2, 'PUSH1', 2, 'PUSH1', 1, 'MULMOD'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 2, 'PUSH1', 2, 'PUSH1', 2, 'MULMOD'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0, 'PUSH1', 1, 'PUSH1', 1, 'MULMOD'])
        self.assert_stack([0])

    def test_exp(self):
        self.run_code(['PUSH1', 3, 'PUSH1', 7, 'EXP'])
        self.assert_stack([343])

    def test_signextend(self):
        self.run_code(['PUSH1', 0xff, 'PUSH1', 0x0, 'SIGNEXTEND'])
        self.assert_stack([2**256 - 1])
        self.run_code(['PUSH1', 0x3, 'PUSH1', 0x0, 'SIGNEXTEND'])
        self.assert_stack([3])
        self.run_code(['PUSH1', 0x0, 'PUSH1', 0x0, 'SIGNEXTEND'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0xfffe, 'PUSH1', 0x1, 'SIGNEXTEND'])
        self.assert_stack([2**256 - 2])
        self.run_code(['PUSH1', 0xfffe, 'PUSH1', 0x2, 'SIGNEXTEND'])
        self.assert_stack([0xfffe])

    def test_lt(self):
        self.run_code(['PUSH1', 7, 'PUSH1', 6, 'LT'])
        self.assert_stack([True])
        self.run_code(['PUSH1', 7, 'PUSH1', 7, 'LT'])
        self.assert_stack([False])
        self.run_code(['PUSH1', -7, 'PUSH1', -6, 'LT'])
        self.assert_stack([False])
        self.run_code(['PUSH1', -1, 'PUSH1', 1, 'LT'])
        self.assert_stack([True])

    def test_gt(self):
        self.run_code(['PUSH1', 7, 'PUSH1', 6, 'GT'])
        self.assert_stack([False])
        self.run_code(['PUSH1', 7, 'PUSH1', 7, 'GT'])
        self.assert_stack([False])
        self.run_code(['PUSH1', -7, 'PUSH1', -6, 'GT'])
        self.assert_stack([True])
        self.run_code(['PUSH1', -1, 'PUSH1', 1, 'GT'])
        self.assert_stack([False])

    def test_slt(self):
        self.run_code(['PUSH1', 7, 'PUSH1', 6, 'SLT'])
        self.assert_stack([True])
        self.run_code(['PUSH1', 7, 'PUSH1', 7, 'SLT'])
        self.assert_stack([False])
        self.run_code(['PUSH1', -7, 'PUSH1', -6, 'SLT'])
        self.assert_stack([False])
        self.run_code(['PUSH1', -1, 'PUSH1', 1, 'SLT'])
        self.assert_stack([False])

    def test_sgt(self):
        self.run_code(['PUSH1', 7, 'PUSH1', 6, 'SGT'])
        self.assert_stack([False])
        self.run_code(['PUSH1', 7, 'PUSH1', 7, 'SGT'])
        self.assert_stack([False])
        self.run_code(['PUSH1', -7, 'PUSH1', -6, 'SGT'])
        self.assert_stack([True])
        self.run_code(['PUSH1', -1, 'PUSH1', 1, 'SGT'])
        self.assert_stack([True])

    def test_eq(self):
        self.run_code(['PUSH1', -1, 'PUSH1', 2**256 - 1, 'EQ'])
        self.assert_stack([True])
        self.run_code(['PUSH1', -2, 'PUSH1', 2**256 - 1, 'EQ'])
        self.assert_stack([False])
        self.run_code(['PUSH1', claripy.BoolV(False), 'PUSH1', claripy.BoolV(False), 'EQ'])
        self.assert_stack([True])
        self.run_code(['PUSH1', claripy.BoolV(True), 'PUSH1', claripy.BoolV(False), 'EQ'])
        self.assert_stack([False])

    def test_iszero(self):
        self.run_code(['PUSH1', 1, 'ISZERO'])
        self.assert_stack([False])
        self.run_code(['PUSH1', 1, 'PUSH1', -1, 'ADD', 'ISZERO'])
        self.assert_stack([True])

    def test_and(self):
        self.run_code(['PUSH1', 3, 'PUSH1', 2, 'AND'])
        self.assert_stack([2])

    def test_or(self):
        self.run_code(['PUSH1', 3, 'PUSH1', 2, 'OR'])
        self.assert_stack([3])

    def test_or(self):
        self.run_code(['PUSH1', claripy.BoolV(True), 'PUSH1', 2, 'OR'])
        self.assert_stack([3])

    def test_xor(self):
        self.run_code(['PUSH1', 3, 'PUSH1', 2, 'XOR'])
        self.assert_stack([1])

        a = random.randint(0, 2**256 - 1)
        b = random.randint(0, 2**256 - 1)
        self.run_code(['PUSH1', a, 'PUSH1', b, 'DUP1', 'XOR', 'XOR'])
        self.assert_stack([a])

    def test_not(self):
        self.run_code(['PUSH1', 0x43, 'NOT'])
        self.assert_stack([(2**256 - 1) ^ 0x43])
        self.run_code(['PUSH1', 0, 'NOT'])
        self.assert_stack([2**256 - 1])
        self.run_code(['PUSH1', 2**256 - 1, 'NOT'])
        self.assert_stack([0])

        a = random.randint(0, 2**256 - 1)
        self.run_code(['PUSH1', a, 'NOT', 'NOT'])

    def test_byte(self):
        self.run_code(['PUSH1', 0x4243, 'PUSH1', 0, 'BYTE'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0x4243, 'PUSH1', 29, 'BYTE'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0x4243, 'PUSH1', 30, 'BYTE'])
        self.assert_stack([0x42])
        self.run_code(['PUSH1', 0x4243, 'PUSH1', 31, 'BYTE'])
        self.assert_stack([0x43])
        self.run_code(['PUSH1', 0x4243, 'PUSH1', 32, 'BYTE'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0x4243, 'PUSH1', 50, 'BYTE'])
        self.assert_stack([0])
        self.run_code(['PUSH1', 0x414243, 'PUSH1', 29, 'BYTE'])
        self.assert_stack([0x41])

    def test_pc(self):
        self.run_code(['PUSH1', 1, 'PC'])
        self.assert_stack([1, 2])

    def test_gas(self):
        self.run_code(['GAS'])
        self.assert_stack([self.state.env.gas])

    def test_address(self):
        self.run_code(['ADDRESS'])
        self.assert_stack([self.state.env.address])

    def test_balance(self):
        self.run_code(['ADDRESS', 'BALANCE'],
                      env={'address': claripy.BVV(789, 256)})
        self.assert_stack([self.state.env.balance])

        with self.assertRaises(ValueError):
            self.run_code(['PUSH1', 1, 'BALANCE'])

    def test_origin(self):
        self.run_code(['ORIGIN'])
        self.assert_stack([self.state.env.origin])

    def test_caller(self):
        self.run_code(['CALLER'])
        self.assert_stack([self.state.env.caller])

    def test_callvalue(self):
        self.run_code(['CALLVALUE'])
        self.assert_stack([self.state.env.value])

    def test_blockhash(self):
        self.run_code(['NUMBER', 'BLOCKHASH'])
        self.assert_stack([
            self.state.env.block_hashes[self.state.env.block_number]])

    def test_timestamp(self):
        self.run_code(['TIMESTAMP'])
        self.assert_stack([self.state.env.block_timestamp])

    def test_number(self):
        self.run_code(['NUMBER'])
        self.assert_stack([self.state.env.block_number])

    def test_coinbase(self):
        self.run_code(['COINBASE'])
        self.assert_stack([self.state.env.coinbase])

    def test_difficulty(self):
        self.run_code(['DIFFICULTY'])
        self.assert_stack([self.state.env.difficulty])

    def test_pop(self):
        self.run_code(['PUSH1', 1, 'POP'])
        self.assert_stack([])
        with self.assertRaises(utils.CodeError):
            self.run_code(['POP'])

    def test_jump(self):
        r = self.run_code(['PUSH1', 4, 'JUMP', 'PC', 'JUMPDEST'])
        self.assertFalse(r)
        self.assert_stack([])
        self.assertEqual(len(self.sm.branch_queue), 2)
        self.sm.exec_branch(self.sm.branch_queue[1][1])
        self.assert_stack([])

        with self.assertRaises(utils.CodeError):
            self.run_code(['PUSH1', 4, 'JUMP', 'POP', 'PC'])

    def test_jumpi(self):
        r = self.run_code(['PUSH1', 1337, 'CALLVALUE', 'EQ',
                           'PUSH1', 8,
                           'JUMPI',
                           'PC',
                           'JUMPDEST'])
        self.assertFalse(r)

        def true_state(state):
            self.state = state
            self.sm.exec_branch(self.state)
            try:
                self.assert_stack([7])
                self.assertEqual(self.state.solver.min(self.state.env.value), 0)
            except AssertionError:
                return False
            return True

        def false_state(state):
            self.state = state
            self.sm.exec_branch(self.state)
            try:
                self.assert_stack([])
                self.assertEqual(self.state.solver.min(self.state.env.value), 1337)
            except AssertionError:
                return False
            return True

        # We don't know which state is first (it changes). So we do that
        # to avoid flakiness.
        self.assertTrue(any(true_state(bq[1])) for bq in self.sm.branch_queue)
        self.assertTrue(any(false_state(bq[1])) for bq in self.sm.branch_queue)

        with self.assertRaises(utils.CodeError):
            self.run_code(['PUSH1', 0, 'PUSH1', 0, 'JUMPI'])


    def test_push(self):
        self.run_code(['PUSH1', 2])
        self.assert_stack([2])
        self.run_code(['PUSH3', 2, 0, 0, 'PUSH1', 4])
        self.assert_stack([2, 4])

    def test_dup(self):
        self.run_code(['PUSH1', 1, 'PUSH1', 2, 'PUSH1', 3, 'DUP3'])
        self.assert_stack([1, 2, 3, 1])
        self.run_code(['PUSH1', 1, 'PUSH1', 2, 'PUSH1', 3, 'DUP1'])
        self.assert_stack([1, 2, 3, 3])

    def test_swap(self):
        self.run_code(['PUSH1', 1, 'PUSH1', 2, 'PUSH1', 3, 'SWAP2'])
        self.assert_stack([3, 2, 1])
        self.run_code(['PUSH1', 1, 'PUSH1', 2, 'PUSH1', 3, 'SWAP1'])
        self.assert_stack([1, 3, 2])

    def test_log(self):
        self.run_code(['PUSH1', 0, 'PUSH1', 0, 'PUSH1', 0, 'LOG1'])

    # TODO: test_sha3

    def test_stop(self):
        r = self.run_code(['STOP'])
        self.assertTrue(r)

    def test_return(self):
        r = self.run_code(['RETURN'])
        self.assertTrue(r)

    def test_calldataload(self):
        self.run_code(['PUSH1', 0, 'CALLDATALOAD', 'PUSH1', 0, 'CALLDATALOAD'])
        self.assertTrue(self.state.stack[0] is self.state.stack[1])
        self.run_code(['PUSH1', 0, 'CALLDATALOAD', 'PUSH1', 32, 'CALLDATALOAD'])
        self.assertTrue(self.state.stack[0] is not self.state.stack[1])

    def test_calldatasize(self):
        self.run_code(['CALLDATASIZE', 'PUSH1', 4, 'CALLDATALOAD'])
        self.assertEqual(self.state.solver.min(self.state.stack[0]), 36)

    def test_calldatacopy(self):
        # size, dstart, mstart
        self.run_code(['PUSH1', 64, 'PUSH1', 20, 'PUSH1', 10, 'CALLDATACOPY',
                       'PUSH1', 10, 'MLOAD', 'PUSH1', 42, 'MLOAD'])
        self.assertEqual(self.state.solver.min(self.state.env.calldata_size), 84)
        self.assert_stack([self.state.env.calldata.read(20, 32),
                           self.state.env.calldata.read(52, 32)])

    def test_codesize(self):
        self.run_code(['PUSH1', 0, 'POP', 'CODESIZE'])
        self.assert_stack([4])

    def test_extcodesize_self(self):
        self.run_code(['ADDRESS', 'EXTCODESIZE'])
        self.assert_stack([2])

    def test_extcodesize_other(self):
        self.run_code(['PUSH1', 42, 'EXTCODESIZE'])
        self.assertTrue(len(self.state.stack), 1)
        self.assertTrue(self.state.solver.satisfiable(
            extra_constraints=[self.state.stack[0] == 0]))
        self.assertTrue(self.state.solver.satisfiable(
            extra_constraints=[self.state.stack[0] == 1337]))

    @mock.patch.object(utils, 'disassemble')
    def test_codecopy(self, mock_disassemble):
        code = ['CODESIZE', 'PUSH1', 0, 'DUP1', 'CODECOPY',
                'PUSH1', 0, 'MLOAD']
        mock_disassemble.return_value = code
        self.sm = sm.SymbolicMachine(Env(
            b'\x60' + bytearray(len(code) - 2) + b'\xa1'))
        self.state = self.sm.branch_queue[0][1]
        self.sm.exec_branch(self.state)
        number = 0x60000000000000a1 << ((32-len(code))*8)
        self.assert_stack([number])

    def test_mload(self):
        self.run_code(['PUSH1', 0, 'MLOAD', 'PUSH1', 0, 'MLOAD'])
        self.assertTrue(self.state.stack[0] is self.state.stack[1])

    def test_mstore(self):
        self.run_code(['PUSH1', 1337, 'PUSH1', 1, 'MSTORE', 'PUSH1', 1, 'MLOAD'])
        self.assert_stack([1337])
        self.run_code(['PUSH1', 1337, 'PUSH1', 0, 'MSTORE', 'PUSH1', 0, 'MLOAD'])
        self.assert_stack([1337])
        self.run_code(['PUSH1', 0xcafe, 'PUSH1', 64, 'MSTORE',
                       'PUSH1', 0xdead, 'PUSH1', 62, 'MSTORE',
                       'PUSH1', 66, 'MLOAD'])
        self.assert_stack([0xdeadcafe0000])

    def test_mstore8(self):
        self.run_code(['PUSH1', 0x1337, 'PUSH1', 30, 'MSTORE8', 'PUSH1', 0, 'MLOAD'])
        self.assert_stack([0x3700])

    def test_msize(self):
        self.run_code(['PUSH1', 1, 'PUSH1', 0, 'MSTORE8', 'MSIZE'])
        self.assert_stack([1])
        self.run_code(['PUSH1', 1, 'PUSH1', 32, 'MSTORE', 'MSIZE'])
        self.assert_stack([64])

    def test_sload(self):
        self.run_code(['PUSH1', 0, 'SLOAD', 'PUSH1', 0, 'SLOAD'])
        self.assertTrue(self.state.stack[0] is self.state.stack[1])
        self.run_code(['PUSH1', 0, 'SLOAD', 'PUSH1', 32, 'SLOAD'])
        self.assertTrue(self.state.stack[0] is not self.state.stack[1])

    def test_sstore(self):
        self.run_code(['PUSH1', 1337, 'PUSH1', 0, 'SSTORE', 'PUSH1', 0, 'SLOAD',
                       'PUSH1', 1338, 'PUSH1', 0, 'SSTORE', 'PUSH1', 0, 'SLOAD'])
        self.assert_stack([1337, 1338])
        self.run_code(['PUSH1', 0, 'SLOAD', 'PUSH1', 32, 'SSTORE',
                       'PUSH1', 32, 'SLOAD',
                       'PUSH1', 0, 'SLOAD'])
        self.assertTrue(self.state.stack[0] is self.state.stack[1])

    # TODO: test_call, test_callcode, test_suicide

    def test_invalid_opcode(self):
        with self.assertRaises(utils.CodeError):
            self.run_code(['INVALID 0xfe'])

if __name__ == '__main__':
    unittest.main()



import unittest
import glob
import os
import logging
import json
import subprocess
import codecs
import collections
import shutil
import sys

from web3 import Web3

from pakala import utils
from pakala import sm
from pakala import env
from pakala import recursive_analyzer
from pakala import analyzer

logger = logging.getLogger(__name__)

MAX_TO_SEND = Web3.toWei(1000, 'ether')
MIN_TO_RECEIVE = Web3.toWei(1, 'wei')
EXEC_TIMEOUT = 1*60
ANALYSIS_TIMEOUT = 30*60
MAX_TRANSACTION_DEPTH = 4
ADDRESS = '0xDEADBEEF00000000000000000000000000000000'
BALANCE = Web3.toWei(1, 'ether')


class SolidityTest(unittest.TestCase):
    def __init__(self, filename):
        super().__init__()
        self.filename = filename

    def shortDescription(self):
        return os.path.basename(self.filename)

    @unittest.skipIf(shutil.which("solc") is None, "solc compiler not installed.")
    def runTest(self):
        logger.info("Compiling contract %s" % self.filename)
        p = subprocess.run(
            ["solc", "--optimize", "--combined-json=bin-runtime", self.filename],
            capture_output=True, text=True)
        self.assertEqual(p.returncode, 0,
                         "solc compilation failed:\n%s" % p.stderr)

        output = json.loads(p.stdout)

        assert "contracts" in output
        identifier, properties = list(output['contracts'].items())[0]
        bin_runtime = properties['bin-runtime']

        bin_runtime = codecs.decode(bin_runtime, 'hex')
        logger.info("Compiled. Symbolic execution.")

        e = env.Env(bin_runtime,
                    address=utils.bvv(int(ADDRESS, 16)),
                    caller=utils.DEFAULT_CALLER,
                    origin=utils.DEFAULT_CALLER,
                    balance=utils.bvv(BALANCE))
        s = sm.SymbolicMachine(e)
        s.execute(timeout_sec=EXEC_TIMEOUT)

        self.assertTrue(s.outcomes)

        ra = recursive_analyzer.RecursiveAnalyzer(
                max_wei_to_send=MAX_TO_SEND,
                min_wei_to_receive=MIN_TO_RECEIVE,
                block='invalid')

        # Never contact the blockchain, instead all the storage are 0
        ra.storage_cache = analyzer.EmptyStorage()

        bug = ra.check_states(
                s.outcomes,
                timeout=ANALYSIS_TIMEOUT,
                max_depth=MAX_TRANSACTION_DEPTH)

        self.assertTrue(bug, self.filename)


def load_tests(loader, tests, pattern):
    files = glob.glob(os.path.join(os.path.dirname(__file__), "*.sol"))
    return unittest.TestSuite(map(SolidityTest, files))


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    logging.getLogger('claripy').setLevel(logging.INFO)
    if len(sys.argv) == 1:
        unittest.main()
    else:
        files = sys.argv[1:]
        for filename in files:
            SolidityTest(filename).runTest()



import logging
import itertools
import collections
import random

from claripy import frontend_mixins
from claripy import frontends
from claripy import backends
from claripy.ast import bv
import claripy

logger = logging.getLogger(__name__)

# TODO: we could keep the resulting constraints in cache or something, so it's
# very fast when we don't use the solver with non-empty extra_constraint.

def Sha3(x):
    return bv.BV('SHA3', [x], length=256)


def _symbolize_hashes(ast, hashes):
    if not isinstance(ast, claripy.ast.base.Base):
        return ast

    # Replace SHA3 with a BVS
    if ast.op == 'SHA3':
        hash_input, = ast.args
        hash_input = _symbolize_hashes(hash_input, hashes)
        try:
            return hashes[hash_input]
        except KeyError:
            hash_symbol = claripy.BVS('SHA3', 256)
            hashes[hash_input] = hash_symbol
            logger.debug("Registering new hash: %s(%s)",
                         hash_symbol, hash_input)
            return hash_symbol

    # Recursively apply to children
    args = [_symbolize_hashes(child, hashes) for child in ast.args]
    return ast.swap_args(args)


def _no_sha3_symbol(ast):
    if not isinstance(ast, claripy.ast.base.Base):
        return True
    elif isinstance(ast, claripy.ast.base.BV):
        try:
            return not ast.args[0].startswith('SHA3')
        except AttributeError:
            return True
    else:
        return all(_no_sha3_symbol(child) for child in ast.args)


def _no_sha3_symbols(constraints):
    return all(_no_sha3_symbol(ast) for ast in constraints)


class Sha3Mixin(object):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.hashes = {}  # Mapping hash input to the symbol

    def _copy(self, c):
        super()._copy(c)
        c.hashes = self.hashes.copy()

    def _blank_copy(self, c):
        super()._blank_copy(c)
        c.hashes = {}

    def add(self, constraints, **kwargs):
        if isinstance(constraints, claripy.ast.base.Base):
            constraints = [constraints]
        # TODO: Put the assertion here. Problem is that this is called from
        # inside claripy as well.
        #assert _no_sha3_symbols(constraints)
        constraints = [_symbolize_hashes(c, self.hashes) for c in constraints]
        return super().add(constraints, **kwargs)

    def satisfiable(self, extra_constraints=(), **kwargs):
        # TODO: Put the assertion here. Problem is that this is called from
        # inside claripy as well.
        #assert _no_sha3_symbols(extra_constraints)
        try:
            extra_constraints = self._hash_constraints(
                    extra_constraints, hashes=self.hashes.copy())
        except claripy.errors.UnsatError:
            return False
        return super().satisfiable(extra_constraints=extra_constraints)

    def eval(self, e, n, extra_constraints=(), **kwargs):
        assert _no_sha3_symbol(e)
        assert _no_sha3_symbols(extra_constraints)
        hashes = self.hashes.copy()
        e = _symbolize_hashes(e, hashes)
        extra_constraints = self._hash_constraints(
            extra_constraints, hashes=hashes)
        return super().eval(e, n, extra_constraints=extra_constraints)

    def batch_eval(self, e, n, extra_constraints=(), **kwargs):
        raise NotImplementedError()

    def max(self, e, extra_constraints=(), **kwargs):
        assert _no_sha3_symbol(e)
        assert _no_sha3_symbols(extra_constraints)
        hashes = self.hashes.copy()
        e = _symbolize_hashes(e, hashes)
        extra_constraints = self._hash_constraints(
            extra_constraints, hashes=hashes)
        return super().max(e, extra_constraints=extra_constraints)

    def min(self, e, extra_constraints=(), **kwargs):
        assert _no_sha3_symbol(e)
        assert _no_sha3_symbols(extra_constraints)
        hashes = self.hashes.copy()
        e = _symbolize_hashes(e, hashes)
        extra_constraints = self._hash_constraints(
            extra_constraints, hashes=hashes)
        return super().min(e, extra_constraints=extra_constraints)

    def _hash_constraints(self, extra_constraints,
                          hashes, pairs_done=None):
        extra_constraints = [
                _symbolize_hashes(c, hashes) for c in extra_constraints]

        # Fast-path if no hashes, or if not satisfiable.
        if not hashes or not super().satisfiable(extra_constraints=extra_constraints):
            return tuple(extra_constraints)

        if pairs_done is None:
            pairs_done = set()

        new_extra_constraints = []
        for (in1, s1), (in2, s2) in itertools.combinations(hashes.items(), 2):
            if (s1, s2) in pairs_done:
                continue
            # Do s1 needs to be equal to s2 ? Then in1 needs to be equal to in2
            if not super().satisfiable(extra_constraints=extra_constraints + [s1 != s2]):
                new_extra_constraints.append(in1 == in2)
                logger.debug("Added input constraint: %s", in1 == in2)
                pairs_done.add((s1, s2))
                pairs_done.add((s2, s1))
            # Do s1 needs to be != to s2 ? Then in1 needs to be != to in2
            elif not super().satisfiable(extra_constraints=extra_constraints + [s1 == s2]):
                new_extra_constraints.append(in1 != in2)
                logger.debug("Added input constraint: %s", in1 != in2)
                pairs_done.add((s1, s2))
                pairs_done.add((s2, s1))

        if new_extra_constraints:
            return self._hash_constraints(
                extra_constraints + new_extra_constraints, hashes, pairs_done)

        assert super().satisfiable(extra_constraints=extra_constraints)

        for in1, s1 in hashes.items():
            # Next line can raise UnsatError. Handled in the caller if needed.
            sol1, = super().eval(in1, 1, extra_constraints=extra_constraints)
            extra_constraints.append(in1 == sol1)
            # TODO: use actual hash value! Not this pseudo-hash thing.
            random.seed(sol1)
            extra_constraints.append(s1 == random.randint(0, 2**256 - 1))
            logger.debug("Added concrete constraint: %s", extra_constraints[-1])

        return tuple(extra_constraints)

    def replace(self, r):
        # First replacement: apply r() everywhere.
        self.constraints = [r(i) for i in self.constraints]
        self.hashes = {r(k): r(v) for k, v in self.hashes.items()}
        # Second one: change the hash symbols as well.
        # Also needed in case we re-import constraints before the replacement:
        # the input is different and we don't want the symbols to be the same.
        new_hashes = {k: claripy.BVS('SHA3', 256)
                      for k, v in self.hashes.items()}
        for k in self.hashes:
            r_from, r_to = self.hashes[k], new_hashes[k]
            self.constraints = [i.replace(r_from, r_to)
                                for i in self.constraints]
        self.hashes = new_hashes

        self.downsize()  # Half-assed attempt at clearing caches... TODO improve.

    def combine(self, others):
        combined = super().combine(others)
        combined.hashes.update(self.hashes)

        for other in others:
            for k, v in self.hashes.items():
                # Make sure the symbols are distinct
                assert all(v is not v2 for v2 in other.hashes.values())
                # TODO: Support identical inputs. We just have to make the symbols identical.
                assert all(k is not k2 for k2 in other.hashes.keys())
            combined.hashes.update(other.hashes)

        return combined

    def __repr__(self):
        return "ClaripySha3(constraints=%s, hashes=%s)" % (
            self.constraints,
            self.hashes)

    def as_dict(self):
        return {'constraints': self.constraints,
                'hashes': self.hashes}


class Solver(
    Sha3Mixin,
    frontend_mixins.ConstraintFixerMixin,
    frontend_mixins.ConcreteHandlerMixin,
    frontend_mixins.EagerResolutionMixin,
    frontend_mixins.ConstraintFilterMixin,
    frontend_mixins.ConstraintDeduplicatorMixin,
    frontend_mixins.SimplifySkipperMixin,
    frontend_mixins.SatCacheMixin,
    frontend_mixins.ModelCacheMixin,
    frontend_mixins.ConstraintExpansionMixin,
    frontend_mixins.SimplifyHelperMixin,
    frontends.FullFrontend):

    def __init__(self, backend=backends.z3, **kwargs):
        super().__init__(backend, **kwargs)



import claripy
import unittest
import functools

from pakala.state import State
from pakala import env


class TestState(unittest.TestCase):
    def testHashWorks(self):
        state = State(env.Env(b''))
        state.pc = 5
        state.memory.write(0, 1, claripy.BVV(42, 8))
        state.memory.write(10, 1, claripy.BVV(43, 8))
        state.memory.write(20, 1, claripy.BVV(44, 8))
        state_copy = state.copy()
        self.assertEqual(hash(state), hash(state_copy))

        state.pc = 6
        self.assertNotEqual(hash(state), hash(state_copy))
        state_copy.pc = 6
        self.assertEqual(hash(state), hash(state_copy))

        state.memory.write(10, 1, claripy.BVV(45, 8))
        self.assertNotEqual(hash(state), hash(state_copy))
        state_copy.memory.write(10, 1, claripy.BVV(45, 8))
        self.assertEqual(hash(state), hash(state_copy))

        state.stack_push(state.env.calldata.read(0, 1))
        self.assertNotEqual(hash(state), hash(state_copy))
        state_copy.stack_push(state_copy.env.calldata.read(0, 1))
        self.assertEqual(hash(state), hash(state_copy))

    def testSameHashIfDifferentOrder(self):
        a = State()
        b = State()
        self.assertEqual(hash(a), hash(b))

        e = env.Env("code")

        a.solver.add(e.value == 1)
        a.solver.add(e.block_timestamp == 2)

        # Same thing, different order
        b.solver.add(e.block_timestamp == 2)
        b.solver.add(e.value == 1)

        self.assertEqual(hash(a), hash(b))

    def testReplace(self):
        old_env = env.Env(b'')
        new_env = old_env.clean_copy()
        state = State(new_env)
        state.storage_written[old_env.caller] = old_env.value
        state.replace(functools.partial(env.replace, old_env, state.env))
        self.assertIs(state.storage_written[new_env.caller], new_env.value)



import re
import numbers
import logging

import claripy

# This is toxic material, as it can mess up with the logging configuration with
# its "slogging" module.
from ethereum import opcodes
from ethereum import utils
from ethereum import vm
logging._loggerClass = logging.Logger  # Counter-hack to fix the logging monkey-patch.

from pakala import claripy_sha3

# Custom logging levels:
INFO_INTERACTIVE = 19
logging.addLevelName(INFO_INTERACTIVE, "INFO_INTERACTIVE")

ADDR_MASK = 0xffffffffffffffffffffffffffffffffffffffff

DEFAULT_ADDRESS = claripy.BVV(0xcafebabeffffffffffffffffffffff7cff7247c9, 256)
DEFAULT_CALLER = claripy.BVV(0xcafebabefffffffff0202fffffffff7cff7247c9, 256)


class CodeError(Exception):
    pass


class InterpreterError(Exception):
    def __init__(self, state, message):
        self.state = state
        super().__init__(message)


def bvv(v):
    return claripy.BVV(v, 256)


def get_solver():
    return claripy_sha3.Solver()


def int_to_bytes(v, size):
    b = b''
    while v:
        b = bytearray((v % 256,)) + b
        v = v // 256
    if len(b) < size:
        b = bytearray(size - len(b)) + b
    assert len(b) == size
    return b


def bvv_to_number(bvv):
    if bvv.symbolic:
        raise ValueError("Passed a BVS in bvv_to_number")
    assert isinstance(bvv.args[0], numbers.Number)
    return bvv.args[0]


def disassemble(ocode):
    assert isinstance(ocode, bytes)
    o, pushcache = vm.preprocess_code(ocode)
    code = []
    push_data = 0
    i = 0
    while i < len(ocode):
        try:
            # opcodes[] is like ['opcode', nb_input, nb_output, gas, opcode, param]
            instr = opcodes.opcodes[utils.safe_ord(ocode[i])][0]
        except KeyError:
            instr = 'INVALID 0x%x' % utils.safe_ord(ocode[i])
        if instr.startswith('PUSH'):
            code.append(instr)
            code.append(pushcache[i])
            for x in range(int(instr[4:]) - 1):
                code.append(0)
            i += int(instr[4:])
        else:
            code.append(instr)
        i += 1
    # cde is like ['opcode', 'opcode', 'push', 87, 'opcode', ...]
    return code


def hex_to_bytes(code):
    code = code.strip()
    if not re.match('^(0x)?[0-9a-zA-Z]*$', code):
        raise ValueError("Invalid code.")
    if code.startswith('0x'):
        code = code[2:]
    groups = [code[n:n+2] for n in range(0, len(code), 2)]
    # The bytes(bytearray()) is for python2-compatibility.
    return bytes(bytearray(int(i, 16) for i in groups if len(i) == 2))


def number_to_address(number):
    return '{:#042x}'.format(number)



import claripy
import unittest
import logging
import random
import itertools

from pakala.recursive_analyzer import RecursiveAnalyzer
from pakala.analyzer import FakeStorage
from pakala.env import Env
from pakala.state import State
from pakala import utils
from pakala.claripy_sha3 import Sha3

from web3 import Web3


logging.basicConfig(level=logging.DEBUG)
logging.getLogger('claripy').setLevel(logging.ERROR)


class TestCheckStates(unittest.TestCase):
    """The interesting tests.

    Inventing various classic scenarios and making sure that we find the bug
    if there is one. And that we don't if we are not supposed to find one.
    """

    def setUp(self):
        self.env = Env(b'', caller=utils.DEFAULT_CALLER,
                            address=utils.DEFAULT_ADDRESS)

    def check_states(self, states, mock_storage=None):
        self.analyzer = RecursiveAnalyzer(
            max_wei_to_send=Web3.toWei(10, 'ether'),
            min_wei_to_receive=Web3.toWei(1, 'milliether'))
        self.analyzer.storage_cache = FakeStorage(mock_storage or {})
        return self.analyzer.check_states(states, timeout=0, max_depth=4)

    def get_call(self, value, to=None):
        if to is None:
            to = self.env.caller
        return [utils.bvv(0), utils.bvv(0), utils.bvv(0), utils.bvv(0),
                value, to, utils.bvv(0)]


    def test_nothing(self):
        self.assertFalse(self.check_states([]))

    def test_simple(self):
        state = State(self.env)
        self.assertFalse(self.check_states([state]))

    def test_suicide_simple(self):
        state = State(self.env)
        state.suicide_to = self.env.caller
        self.assertTrue(self.check_states([state]))

    def test_call_simple(self):
        state = State(self.env)
        state.calls.append(self.get_call(self.env.balance))
        self.assertTrue(self.check_states([state]))

    def test_write_and_suicide(self):
        state = State(self.env)

        state_write = state.copy()
        state_write.storage_written = {utils.bvv(0):
                                       self.env.calldata.read(4, 32)}

        state_suicide = state.copy()
        state_suicide.suicide_to = self.env.calldata.read(4, 32)
        storage_0 = claripy.BVS('storage[0]', 256)
        state_suicide.storage_read = {utils.bvv(0): storage_0}
        state_suicide.solver.add(storage_0 == 0xDEADBEEF0101)

        storage = {0: 0xBAD1DEA}
        self.assertTrue(self.check_states([state_write, state_suicide],
                    mock_storage=storage))
        self.assertFalse(self.check_states([state_suicide],
                         mock_storage=storage))
        self.assertFalse(self.check_states([state_write]))

    def test_sha3_key(self):
        """Exercise solidity-like mappings, with the key being a sha3."""
        state = State(self.env)

        state_write = state.copy()
        # Arbitrary write input[1], at SHA3(input[0])
        state_write.storage_written = {
            Sha3(self.env.calldata.read(4, 32)): self.env.calldata.read(36, 32)}

        # Needs that: storage[SHA3(input[0])] == 43, made possible by the previous call
        state_suicide = state.copy()
        state_suicide.suicide_to = self.env.calldata.read(36, 32)
        storage_input = claripy.BVS('storage[SHA3(input)]', 256)
        state_suicide.storage_read = {
            Sha3(self.env.calldata.read(4, 32)): storage_input}
        state_suicide.solver.add(storage_input == 0xDEADBEEF101010)

        storage = {55186156870478567193644641351382124067713781048612400765092754877653207859685: 0}
        self.assertTrue(self.check_states([state_write, state_suicide],
                        mock_storage=storage))
        self.assertFalse(self.check_states([state_suicide],
                        mock_storage=storage))
        self.assertFalse(self.check_states([state_write]))

    def test_sha3_value1(self):
        """Exercise comparison of two SHA3 (as values)."""
        state = State(self.env)

        state_write = state.copy()
        state_write.storage_written = {
            utils.bvv(0): Sha3(self.env.calldata.read(4, 32))}

        state_suicide = state.copy()
        state_suicide.suicide_to = self.env.calldata.read(36, 32)
        storage_input = claripy.BVS('storage[0]', 256)
        state_suicide.storage_read = {utils.bvv(0): storage_input}
        state_suicide.solver.add(
                storage_input == Sha3(self.env.calldata.read(4, 32)))

        storage = {0: 0}
        self.assertTrue(self.check_states([state_write, state_suicide],
                        mock_storage=storage))
        self.assertFalse(self.check_states([state_suicide],
                         mock_storage=storage))
        self.assertFalse(self.check_states([state_write],
                         mock_storage=storage))

    def test_sha3_value2(self):
        """Same as above, but we need to pass the computed SHA3."""
        state = State(self.env)

        state_write = state.copy()
        state_write.storage_written = {
            utils.bvv(0): Sha3(self.env.calldata.read(4, 32))}

        state_suicide = state.copy()
        state_suicide.suicide_to = self.env.calldata.read(36, 32)
        storage_input = claripy.BVS('storage[0]', 256)
        state_suicide.storage_read = {utils.bvv(0): storage_input}
        state_suicide.solver.add(storage_input == self.env.calldata.read(4, 32))
        state_suicide.solver.add(storage_input != 0)

        storage = {0: 0}
        self.assertTrue(self.check_states([state_write, state_suicide],
                        mock_storage=storage))
        self.assertFalse(self.check_states([state_suicide],
                         mock_storage=storage))
        self.assertFalse(self.check_states([state_write],
                         mock_storage=storage))

    def test_write_write_and_suicide(self):
        state = State(self.env)
        # Anybody can set owner
        state_write1 = state.copy()
        state_write1.storage_written = {
                utils.bvv(0): self.env.calldata.read(4, 32)}

        # Onlyowner: set a magic constant allowing the suicide bug, at an
        # user-controlled storage key.
        state_write2 = state.copy()
        read_0 = claripy.BVS('storage[0]', 256)
        state_write2.storage_read = {utils.bvv(0): read_0}
        state_write2.storage_written = {
                self.env.calldata.read(36, 32): self.env.calldata.read(4, 32)}
        state_write2.solver.add(read_0 == self.env.caller)

        # Suicide, when owner and magic constant set
        state_suicide = state.copy()
        read_0 = claripy.BVS('storage[0]', 256)
        read_40 = claripy.BVS('storage[4]', 256)
        state_suicide.storage_read = {
            utils.bvv(0): read_0,
            utils.bvv(40): read_40}
        state_suicide.solver.add(self.env.caller == read_0)
        state_suicide.solver.add(read_40 == 1337)
        state_suicide.suicide_to = self.env.caller

        states = [state_write1, state_write2, state_suicide]
        random.shuffle(states)

        storage = {0: 123456789, 40: 387642}
        for s in itertools.combinations(states, 2):
            self.assertFalse(self.check_states(s, mock_storage=storage))
        self.assertTrue(self.check_states(states, mock_storage=storage))

    def test_send_after_write(self):
        state = State(self.env)

        # We send storage[0]
        state_send = state.copy()
        storage_0 = claripy.BVS('storage[0]', 256)
        state_send.storage_read = {utils.bvv(0): storage_0}
        state_send.calls.append(self.get_call(storage_0))

        # storage[0] is 0.5 ETH
        storage = {0: Web3.toWei(0.5, 'ether')}
        self.assertTrue(self.check_states([state_send], mock_storage=storage))

        # storage[0] is 0 ETH
        storage = {0: 0}
        self.assertFalse(self.check_states([state_send], mock_storage=storage))

        # storage[0] is still 0 ETH initially, but we have an arbitrary write now
        state_write = state.copy()
        state_write.storage_written = {utils.bvv(0):
                                       self.env.calldata.read(4, 32)}
        state_write.solver.add(self.env.calldata.read(0, 4) == 0x1337)
        state_write.solver.add(
                self.env.calldata.read(4, 32) < Web3.toWei(1, 'ether'))

        self.assertFalse(self.check_states([state_write],
                                           mock_storage=storage))
        self.assertTrue(self.check_states([state_send, state_write],
                                          mock_storage=storage))
        self.assertTrue(self.check_states([state_write, state_send],
                                          mock_storage=storage))

        # ...arbitrary write of 1 wei only, which is too little
        state_write_0 = state_write.copy()
        state_write_0.solver.add(self.env.calldata.read(4, 32) == 1)
        self.assertFalse(self.check_states([state_write_0, state_send],
                                           mock_storage=storage))

        # ...arbitrary write only if the block timestamp is <10, which is impossible.
        state_write_ts = state_write.copy()
        state_write_ts.solver.add(self.env.block_timestamp < 10)
        self.assertFalse(self.check_states([state_write_ts, state_send],
                                           mock_storage=storage))

        self.assertFalse(self.check_states(
            [state_write_0, state_send, state_write_ts],
            mock_storage=storage))

        # now we put all these state_write* together, so there is a solution.
        self.assertTrue(self.check_states(
            [state_write_0, state_send, state_write, state_write_ts],
            mock_storage=storage))
        self.assertTrue(self.check_states(
            [state_write_0, state_write, state_write_ts, state_send],
            mock_storage=storage))



import collections
import numbers
import time

import claripy

from pakala import memory
from pakala import utils

ENV_VARS = (
    ('caller', None, None),
    ('origin', None, None),
    ('value', None, 10**20),
    ('address', None, None),
    ('balance', None, None),
    ('gas', None, None),
    ('block_timestamp', int(time.time()), int(time.time() + 86400*365)),
    ('block_number', 6000000, 10**9),
    ('calldata_size', None, 2**20),
    ('coinbase', None, None),
    ('difficulty', None, None),
)

class Env(object):
    def __init__(self, code, **kwargs):
        self.code = code
        self.calldata = memory.CalldataMemory()
        self.block_hashes = {}

        for name, _, _ in ENV_VARS:
            default = claripy.BVS(name, 256)
            setattr(self, name, kwargs.get(name, default))

    def __repr__(self):
        return "Env(balance=%s, caller=%s, value=%s)" % (
            self.balance, self.caller, self.value)

    def as_dict(self):
        return {'balance': self.balance,
                'caller': self.caller,
                'value': self.value}

    def clean_copy(self):
        """Create a new env, which is a copy of the current one but with
        new symbolic variables (with the same name)"""
        new_env = Env(self.code)

        for name, _, _ in ENV_VARS:
            value = getattr(self, name)
            if value.symbolic:
                setattr(new_env, name, claripy.BVS(name, 256))
            else:
                setattr(new_env, name, value)

        new_env.calldata = self.calldata.copy()
        for addr, value in self.calldata._mem.items():
            new_env.calldata._mem[addr] = (
                claripy.BVS('calldata[%i]' % addr, value.size()))

        # Block hashes are always the same (but the current number can change)
        new_env.block_hashes = self.block_hashes.copy()
        return new_env

    def extra_constraints(self):
        for name, min_, max_ in ENV_VARS:
            if min_ is not None:
                yield getattr(self, name) >= min_
            if max_ is not None:
                yield getattr(self, name) <= max_


def replace(old_env, new_env, var):
    """Replace all the references of old_env to references to new_env, in the
    claripy AST var."""
    for name, _, _ in ENV_VARS:
        var = var.replace(getattr(old_env, name), getattr(new_env, name))

    for addr in old_env.calldata._mem.keys():
        var = var.replace(old_env.calldata._mem[addr],
                          new_env.calldata._mem[addr])

    return var



import claripy
import unittest

from pakala import env


class TestEnv(unittest.TestCase):
    def testReplace(self):
        a = env.Env(b'')
        calldata_a = a.calldata.read(0, 32)
        b = a.clean_copy()
        self.assertIs(env.replace(a, b, a.value), b.value)
        self.assertIs(env.replace(a, b, calldata_a), b.calldata.read(0, 32))



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pakala',
    version='1.0.2',

    description='An EVM symbolic execution tool and vulnerability scanner',

    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/palkeo/pakala',

    author='palkeo',

    author_email='ethereum@palkeo.com',

    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Pick your license as you wish
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        'Programming Language :: Python :: 3',
    ],

    keywords='ethereum evm symbolic execution vulnerability scanner',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=['ethereum', 'claripy', 'web3'],

    extras_require={
        'test': ['mock'],
    },
)



"""
    pakala: EVM symbolic execution tool and vulnerability scanner.
    Copyright (C) 2018 Korantin Auguste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import collections
import datetime
import heapq
import logging
import math
import numbers
import time

from ethereum import opcodes
from ethereum import vm
from ethereum import utils as ethutils
import claripy

from pakala import utils
from pakala.state import State
from pakala.claripy_sha3 import Sha3

logger = logging.getLogger(__name__)  # pylint:disable=invalid-name


bvv = utils.bvv  # pylint:disable=invalid-name
BVV_0 = bvv(0)
BVV_1 = bvv(1)

# interesting values aligned to classic parameters.
CALLDATASIZE_FUZZ = [min, max, 4, 32, 36, 64, 68, 100, 132, 164, 196]


def not_bool(variable):
    """Ensure a BV is not a BoolS.
    If it is, it's converted to a BVV: 0 or 1.
    """
    if isinstance(variable, claripy.ast.Bool):
        return claripy.If(variable, BVV_1, BVV_0)
    return variable


def make_consistent(a, b):
    """Ensure a and b are not bool and not bool."""
    if isinstance(a, claripy.ast.Bool) and not isinstance(b, claripy.ast.Bool):
        return not_bool(a), b
    elif isinstance(b, claripy.ast.Bool) and not isinstance(a, claripy.ast.Bool):
        return a, not_bool(b)
    return a, b


class SymbolicMachine:
    """Class to represent a state of a EVM program, and execute it symbolically.
    """
    def __init__(self, env):
        self.code = utils.disassemble(env.code)
        logger.debug("Initializing symbolic machine with source code: %s", self.code)
        # For use by heapq only. Contains couples (score, state).
        self.branch_queue = []
        self.states_seen = set()
        self.coverage = [0]*len(self.code)
        # List of all normal/good terminations of the contract
        self.outcomes = []
        # List of all the place where we didn't know how to continue execution
        self.partial_outcomes = []
        self.fuzz = True
        self.code_errors = collections.Counter()
        self.interpreter_errors = collections.Counter()
        self.add_branch(State(env))

    def add_branch(self, state):
        """Add a state corresponding to a branch to be executed.
        We look at how many times the branch was executed, that's the score.
        We will take the branches with the smallest score first.
        """
        if not state.solver.satisfiable():
            logger.debug("Avoided adding unsatisfiable state.")
            self.code_errors["Avoided adding unsatisfiable state"] += 1
            return

        if hash(state) in self.states_seen:
            logger.debug("Avoided adding visited state.")
            self.code_errors["Avoided adding visited state"] += 1
            return

        # We have to downsize the used solver to free memory in z3, otherwise
        # they will collectively end up eating all the memory.
        state.solver.downsize()
        state.solver.simplify()

        # If it's a unknown code, let's set the score to 0, otherwise use the
        # score from the state.
        new_code = state.pc >= len(self.coverage) or not self.coverage[state.pc]
        if new_code:
            state.score = 0

        # TODO: Run comprehensive tests with that formula instead.
        # state.score = 0 if state.pc >= len(self.coverage) else self.coverage[state.pc]

        logger.debug("Adding branch to %i with score %i "
                     "(visited %i times)", state.pc, state.score,
                     self.coverage[state.pc] if state.pc < len(self.coverage) else 0)

        heapq.heappush(self.branch_queue, (state.score, state))
        self.states_seen.add(hash(state))

    def add_for_fuzzing(self, state, variable, tries):
        """
        Will try to fuzz the variable, setting it to different values.
        The tries parameter must be an array of :
         - min: if you want to add the minimum value possible
         - max: if you want to add the maximum value possible
         - a number: to try that number
         - None: a random number that works, you can repeat it.
        """
        # TODO: If the fuzzer is used, then I will generate tons of branches
        # that are equivalent... There should be a way to deduplicate them,
        # but it's not trivial.

        if not self.fuzz:
            raise utils.InterpreterError(state, "Fuzzer is disabled")

        to_try = set()
        nb_random = 0
        for t in tries:
            if isinstance(t, numbers.Number) and state.solver.solution(variable, t):
                to_try.add(t)
            elif t is min:
                to_try.add(state.solver.min(variable))
            elif t is max:
                to_try.add(state.solver.max(variable))
            elif t is None:
                nb_random += 1
        if nb_random:
            to_try |= set(state.solver.eval(variable, nb_random))

        logger.debug('Fuzzing will try %s in %s.', variable, to_try)
        for value in to_try:
            new_state = state.copy()
            new_state.solver.add(variable == value)
            self.add_branch(new_state)

    def exec_branch(self, state):  # pylint: disable=too-many-locals,too-many-return-statements,too-many-branches,too-many-statements
        """Execute forward from a state, queuing new states if needed."""
        logger.debug("Constraints: %s", state.solver.constraints)

        def solution(variable):
            """Returns the solution. There must be one or we fail."""
            solutions = state.solver.eval(variable, 2)
            if len(solutions) > 1:
                raise ValueError("Ambiguous solution for %s (%s)" %
                                 (variable, self.code[state.pc]))
            solution = solutions[0]
            return solution if isinstance(solution, numbers.Number) else solution.value

        state.score += 1

        while True:
            if state.pc >= len(self.code):
                return True

            # TODO: Don't log for things like PUSH, POP...
            logger.debug("NEW STEP")
            logger.debug('Memory: %s', state.memory)
            logger.debug('Stack: %s', state.stack)
            logger.debug('PC: %i %s', state.pc, self.code[state.pc])

            op = self.code[state.pc]  # pylint:disable=invalid-name
            self.coverage[state.pc] += 1

            assert all(hasattr(i, 'symbolic') for i in state.stack), \
                    "The stack musty only contains claripy BV's"

            # Trivial operations first

            if isinstance(op, numbers.Number):
                raise utils.CodeError("Trying to execute PUSH data.")
            elif op == 'JUMPDEST':
                pass
            elif op == 'ADD':
                s0, s1 = not_bool(state.stack_pop()), not_bool(state.stack_pop())  # pylint:disable=invalid-name
                state.stack_push(s0 + s1)
            elif op == 'SUB':
                s0, s1 = not_bool(state.stack_pop()), not_bool(state.stack_pop())  # pylint:disable=invalid-name
                state.stack_push(s0 - s1)
            elif op == 'MUL':
                s0, s1 = not_bool(state.stack_pop()), not_bool(state.stack_pop())  # pylint:disable=invalid-name
                state.stack_push(s0 * s1)
            elif op == 'DIV':
                # We need to use claripy.LShR instead of a division if possible,
                # because the solver is bad dealing with divisions, better
                # with shifts. And we need shifts to handle the solidity ABI
                # for function selection.
                s0, s1 = state.stack_pop(), state.stack_pop()  # pylint:disable=invalid-name
                try:
                    s1 = solution(s1)  # pylint:disable=invalid-name
                except ValueError:
                    state.stack_push(claripy.If(s1 == 0, BVV_0, s0 / s1))
                else:
                    if s1 == 0:
                        state.stack_push(BVV_0)
                    elif s1 == 1:
                        state.stack_push(s0)
                    elif s1 & (s1 - 1) == 0:
                        exp = int(math.log(s1, 2))
                        state.stack_push(s0.LShR(exp))
                    else:
                        state.stack_push(s0 / s1)
            elif op == 'SDIV':
                s0, s1 = state.stack_pop(), state.stack_pop()  # pylint:disable=invalid-name
                try:
                    s1 = solution(s1)
                except ValueError:
                    state.stack_push(claripy.If(s1 == 0, BVV_0, s0.SDiv(s1)))
                else:
                    state.stack_push(BVV_0 if s1 == 0 else s0.SDiv(s1))
            elif op == 'MOD':
                s0, s1 = state.stack_pop(), state.stack_pop()  # pylint:disable=invalid-name
                try:
                    s1 = solution(s1)
                except ValueError:
                    state.stack_push(claripy.If(s1 == 0, BVV_0, s0 % s1))
                else:
                    state.stack_push(BVV_0 if s1 == 0 else s0 % s1)
            elif op == 'SMOD':
                s0, s1 = state.stack_pop(), state.stack_pop()  # pylint:disable=invalid-name
                try:
                    s1 = solution(s1)
                except ValueError:
                    state.stack_push(claripy.If(s1 == 0, BVV_0, s0.SMod(s1)))
                else:
                    state.stack_push(BVV_0 if s1 == 0 else s0.SMod(s1))
            elif op =='ADDMOD':
                s0, s1, s2 = state.stack_pop(), state.stack_pop(), state.stack_pop()
                try:
                    s2 = solution(s2)
                except ValueError:
                    state.stack_push(claripy.If(s2 == 0, BVV_0, (s0 + s1) % s2))
                else:
                    state.stack_push(BVV_0 if s2 == 0 else (s0 + s1) % s2)
            elif op =='MULMOD':
                s0, s1, s2 = state.stack_pop(), state.stack_pop(), state.stack_pop()
                try:
                    s2 = solution(s2)
                except ValueError:
                    state.stack_push(claripy.If(s2 == 0, BVV_0, (s0 * s1) % s2))
                else:
                    state.stack_push(BVV_0 if s2 == 0 else (s0 * s1) % s2)
            elif op == 'EXP':
                base, exponent = solution(state.stack_pop()), state.stack_pop()
                if base == 2:
                    state.stack_push(1 << exponent)
                else:
                    exponent = solution(exponent)
                    state.stack_push(claripy.BVV(base ** exponent, 256))
            elif op == 'LT':
                s0, s1 = not_bool(state.stack_pop()), not_bool(state.stack_pop())  # pylint:disable=invalid-name
                state.stack_push(claripy.ULT(s0, s1))
            elif op == 'GT':
                s0, s1 = not_bool(state.stack_pop()), not_bool(state.stack_pop())  # pylint:disable=invalid-name
                state.stack_push(claripy.UGT(s0, s1))
            elif op == 'SLT':
                s0, s1 = not_bool(state.stack_pop()), not_bool(state.stack_pop())  # pylint:disable=invalid-name
                state.stack_push(claripy.SLT(s0, s1))
            elif op == 'SGT':
                s0, s1 = not_bool(state.stack_pop()), not_bool(state.stack_pop())  # pylint:disable=invalid-name
                state.stack_push(claripy.SGT(s0, s1))
            elif op == 'SIGNEXTEND':
                s0, s1 = state.stack_pop(), state.stack_pop()  # pylint:disable=invalid-name
                # s0 is the number of bits. s1 the number we want to extend.
                s0 = solution(s0)
                if s0 <= 31:
                    sign_bit = 1 << (s0 * 8 + 7)
                    state.stack_push(claripy.If(
                        s1 & sign_bit == 0,
                        s1 & (sign_bit - 1),
                        s1 | ((1<<256) - sign_bit)))
                else:
                    state.stack_push(s1)
            elif op == 'EQ':
                s0, s1 = state.stack_pop(), state.stack_pop()
                if isinstance(s0, claripy.ast.Bool) and isinstance(s1, claripy.ast.Bool):
                    state.stack_push(s0 == s1)
                else:
                    state.stack_push(not_bool(s0) == not_bool(s1))
            elif op == 'ISZERO':
                condition = state.stack_pop()
                if isinstance(condition, claripy.ast.Bool):
                    state.stack_push(claripy.Not(condition))
                else:
                    state.stack_push(condition == BVV_0)
            elif op == 'AND':
                s0, s1 = make_consistent(state.stack_pop(), state.stack_pop())
                if isinstance(s0, claripy.ast.Bool) and isinstance(s1, claripy.ast.Bool):
                    state.stack_push(s0 and s1)
                else:
                    state.stack_push(s0 & s1)
            elif op == 'OR':
                s0, s1 = make_consistent(state.stack_pop(), state.stack_pop())
                if isinstance(s0, claripy.ast.Bool) and isinstance(s1, claripy.ast.Bool):
                    state.stack_push(s0 or s1)
                else:
                    state.stack_push(s0 | s1)
            elif op == 'XOR':
                s0, s1 = make_consistent(state.stack_pop(), state.stack_pop())
                state.stack_push(s0 ^ s1)
            elif op == 'NOT':
                state.stack_push(~state.stack_pop())
            elif op == 'BYTE':
                s0, s1 = state.stack_pop(), state.stack_pop()  # pylint:disable=invalid-name
                state.stack_push(s1.LShR(claripy.If(s0 > 31, 32, 31 - s0) * 8) & 0xff)

            elif op == 'PC':
                state.stack_push(bvv(state.pc))
            elif op == 'GAS':
                state.stack_push(state.env.gas)
            elif op == 'ADDRESS':
                state.stack_push(state.env.address)
            elif op == 'BALANCE':
                addr = solution(state.stack_pop())
                if addr != solution(state.env.address):
                    raise utils.InterpreterError(
                        state,
                        "Can only query balance of the current contract for now")
                state.stack_push(state.env.balance)
            elif op == 'ORIGIN':
                state.stack_push(state.env.origin)
            elif op == 'CALLER':
                state.stack_push(state.env.caller)
            elif op == 'CALLVALUE':
                state.stack_push(state.env.value)
            elif op == 'BLOCKHASH':
                block_num = state.stack_pop()
                if block_num not in state.env.block_hashes:
                    state.env.block_hashes[block_num] = claripy.BVS(
                        'blockhash[%s]' % block_num, 256)
                state.stack_push(state.env.block_hashes[block_num])
            elif op == 'TIMESTAMP':
                state.stack_push(state.env.block_timestamp)
            elif op == 'NUMBER':
                state.stack_push(state.env.block_number)
            elif op == 'COINBASE':
                state.stack_push(state.env.coinbase)
            elif op == 'DIFFICULTY':
                state.stack_push(state.env.difficulty)
            elif op == 'POP':
                state.stack_pop()
            elif op == 'JUMP':
                addr = solution(state.stack_pop())
                if addr >= len(self.code) or self.code[addr] != 'JUMPDEST' or addr == state.pc:
                    raise utils.CodeError('Invalid jump (%i)' % addr)
                state.pc = addr
                self.add_branch(state)
                return
            elif op == 'JUMPI':
                addr, condition = solution(state.stack_pop()), state.stack_pop()
                state_false = state.copy()
                if isinstance(condition, claripy.ast.Bool):
                    state.solver.add(condition)
                    state_false.solver.add(claripy.Not(condition))
                else:
                    state.solver.add(condition != 0)
                    state_false.solver.add(condition == 0)
                state_false.pc += 1
                self.add_branch(state_false)
                state.pc = addr
                if state.pc >= len(self.code) or self.code[state.pc] != 'JUMPDEST':
                    raise utils.CodeError('Invalid jump (%i)' % (state.pc - 1))
                self.add_branch(state)
                return
            elif op.startswith('PUSH'):
                pushnum = int(op[4:])
                push_data = self.code[state.pc + 1]
                state.pc += pushnum
                if hasattr(push_data, 'symbolic'):  # If it's already a BV
                    state.stack_push(push_data)
                else:
                    state.stack_push(bvv(push_data))
            elif op.startswith('DUP'):
                depth = int(op[3:])
                state.stack_push(state.stack[-depth])
            elif op.startswith('SWAP'):
                depth = int(op[4:])
                temp = state.stack[-depth - 1]
                state.stack[-depth - 1] = state.stack[-1]
                state.stack[-1] = temp
            elif op.startswith('LOG'):
                depth = int(op[3:])
                mstart, msz = state.stack_pop(), state.stack_pop()  # pylint:disable=unused-variable
                topics = [state.stack_pop() for x in range(depth)]  # pylint:disable=unused-variable
            elif op == 'SHA3':
                start, length = solution(state.stack_pop()), solution(state.stack_pop())
                memory = state.memory.read(start, length)
                state.stack_push(Sha3(memory))
            elif op == 'STOP':
                return True
            elif op == 'RETURN':
                return True

            elif op == 'CALLDATALOAD':
                indexes = state.stack_pop()
                try:
                    index = solution(indexes)
                except ValueError: # Multiple solutions, let's fuzz.
                    state.stack_push(indexes) # restore the stack
                    self.add_for_fuzzing(state, indexes, CALLDATASIZE_FUZZ)
                    return
                state.solver.add(state.env.calldata_size >= index + 32)
                state.stack_push(state.env.calldata.read(index, 32))
            elif op == 'CALLDATASIZE':
                state.stack_push(state.env.calldata_size)
            elif op == 'CALLDATACOPY':
                old_state = state.copy()
                mstart, dstart, size = state.stack_pop(), state.stack_pop(), state.stack_pop()
                mstart, dstart = solution(mstart), solution(dstart)
                try:
                    size = solution(size)
                except ValueError:
                    self.add_for_fuzzing(old_state, size, CALLDATASIZE_FUZZ)
                    return
                state.memory.copy_from(state.env.calldata, mstart, dstart, size)
                state.solver.add(state.env.calldata_size >= dstart + size)
            elif op == 'CODESIZE':
                state.stack_push(bvv(len(self.code)))
            elif op == 'EXTCODESIZE':
                addr = state.stack_pop()
                if (addr == state.env.address).is_true():
                    state.stack_push(bvv(len(self.code)))
                else:
                    # TODO: Improve that... It's clearly not constraining enough.
                    state.stack_push(claripy.BVS('EXTCODESIZE[%s]' % addr, 256))
            elif op == 'CODECOPY':
                mem_start, code_start, size = [
                    solution(state.stack_pop()) for _ in range(3)]
                for i in range(size):
                    if code_start + i < len(state.env.code):
                        state.memory.write(mem_start + i, 1, claripy.BVV(
                            state.env.code[code_start + i], 8))
                    else:
                        state.memory.write(mem_start + i, 1, claripy.BVV(0, 8))

            elif op == 'MLOAD':
                index = solution(state.stack_pop())
                state.stack_push(state.memory.read(index, 32))
            elif op == 'MSTORE':
                index, value = solution(state.stack_pop()), not_bool(state.stack_pop())
                state.memory.write(index, 32, value)
            elif op == 'MSTORE8':
                index, value = solution(state.stack_pop()), not_bool(state.stack_pop())
                state.memory.write(index, 1, value[7:0])
            elif op == 'MSIZE':
                state.stack_push(bvv(state.memory.size()))
            elif op == 'SLOAD':
                # TODO: This is inaccurate, because the storage can change in a single transaction.
                # See commit d98cab834f8f359f01ef805256d179f5529ebe30.
                key = state.stack_pop()
                if key in state.storage_written:
                    state.stack_push(state.storage_written[key])
                else:
                    if key not in state.storage_read:
                        state.storage_read[key] = claripy.BVS('storage[%s]' % key, 256)
                    state.stack_push(state.storage_read[key])
            elif op == 'SSTORE':
                # TODO: This is inaccurate, because the storage can change in a single transaction.
                # See commit d98cab834f8f359f01ef805256d179f5529ebe30.
                key = state.stack_pop()
                value = state.stack_pop()
                state.storage_written[key] = value

            elif op == 'CALL':
                state.pc += 1

                # First possibility: the call fails (always possible with a call stack big enough)
                state_fail = state.copy()
                state_fail.stack_push(claripy.BoolV(False))
                self.add_branch(state_fail)

                # Second possibility: success.
                state.calls.append(state.stack[-7:])

                gas, to_, value, meminstart, meminsz, memoutstart, memoutsz = (  # pylint:disable=unused-variable
                    state.stack_pop() for _ in range(7))

                if solution(memoutsz) != 0:
                    raise utils.InterpreterError(state, "CALL seems to return data")
                if solution(meminsz) != 0:
                    raise utils.InterpreterError(state, "CALL seems to take data")

                state.stack_push(claripy.BoolV(True))
                self.add_branch(state)
                return

            elif op == 'SUICIDE':
                state.suicide_to = state.stack[-1]
                return True

            elif op == 'REVERT':
                return
            elif op.startswith('INVALID '):
                raise utils.CodeError("Invalid opcode at %i: %s" % (state.pc, op))
            else:
                raise utils.InterpreterError(state, "Unknown opcode %s" % op)

            state.pc += 1

    def execute(self, timeout_sec):
        """Run the code, searching for all the interesting outcomes.

        Returns the process time it took to execute."""

        if self.outcomes:
            raise RuntimeError("Already executed.")

        time_start = time.process_time()

        time_last_coverage_increase = time_start
        last_coverage = 0

        while self.branch_queue:

            coverage = sum(bool(c) for c in self.coverage)
            if coverage > last_coverage:
                time_last_coverage_increase = time.process_time()
                logger.log(utils.INFO_INTERACTIVE,
                           "Coverage is now %i.", coverage)
                last_coverage = coverage

            if ((not timeout_sec and time.process_time() - time_last_coverage_increase > max(60, time_last_coverage_increase - time_start))
                 or (timeout_sec and time.process_time() - time_start > timeout_sec)):
                logger.debug("Timeout.")
                self.interpreter_errors["execute timeout"] += 1
                break

            score, state = heapq.heappop(self.branch_queue)

            logger.debug("Executing branch at %i with score %i.", state.pc, score)
            try:
                success = self.exec_branch(state)
            except (utils.CodeError,
                    claripy.errors.UnsatError) as error:
                logger.debug('Code error: %s', error)
                self.code_errors[repr(error)] += 1
            except (utils.InterpreterError,
                    claripy.errors.ClaripyError,
                    ValueError,
                    ZeroDivisionError) as error:
                logger.debug('Interpreter error: %s', error)
                self.interpreter_errors[repr(error)] += 1
                if isinstance(error, utils.InterpreterError):
                    self.add_partial_outcome(error.state)
            else:
                if success:
                    self.add_outcome(state)
                logger.debug("Branch done.")

        # In case of timeouts, we still have unfinished branches in the queue!
        # Add them as partial outcomes.
        while self.branch_queue:
            score, state = heapq.heappop(self.branch_queue)
            self.add_partial_outcome(state)

        logger.info('Analysis finished with %i outcomes (%i interesting), '
                    'coverage is %i%%',
                    len(self.outcomes),
                    sum(int(o.is_interesting()) for o in self.outcomes),
                    int(self.get_coverage() * 100))

        if self.code_errors:
            logger.info('Code errors encountered: %s',
                        self.code_errors.most_common())
        if self.interpreter_errors:
            logger.info('Interpreter errors encountered: %s',
                        self.interpreter_errors.most_common())

        logger.debug('List of outcomes:')
        for outcome in self.outcomes:
            logger.debug(outcome)

    def add_outcome(self, state):
        """Add an outcome to the list."""
        logger.debug("Adding outcome: %r", state)
        state.clean()
        self.outcomes.append(state)

    def add_partial_outcome(self, state):
        """Add an outcome to the list of partial outcomes."""
        logger.debug("Adding partial outcome: %r", state)
        state.clean()
        self.partial_outcomes.append(state)

    def get_coverage(self):
        """Return the ratio of instructions that were executed by the total
        number of instructions."""
        total_lines = 0
        covered_lines = 0
        for pc, instruction in enumerate(self.code):  # pylint:disable=invalid-name
            if instruction == 'JUMPDEST' or isinstance(instruction, numbers.Number):
                continue
            total_lines += 1
            covered_lines += bool(self.coverage[pc])
        return covered_lines / float(total_lines or 1)



import claripy
import unittest
import logging

from pakala.memory import Memory, CalldataMemory
from pakala import utils

logging.basicConfig(level=logging.DEBUG)


class TestMemory(unittest.TestCase):
    def setUp(self):
        self.mem = Memory()

    def assertBEqual(self, a, b):
        self.assertTrue((a == b).is_true(), msg="%s != %s" % (a, b))

    def assertBNotEqual(self, a, b):
        self.assertTrue((a == b).is_false(), msg="%s == %s" % (a, b))

    def test_read_default(self):
        self.assertBEqual(self.mem.read(1, 1), 0)

    def overwrite(self, a, b):
        self.mem.write(a[0], a[1], claripy.BVV(0x42, a[1]*8))
        self.assertBEqual(self.mem.read(a[0], a[1]), 0x42)
        self.mem.write(b[0], b[1], claripy.BVV(0x01020304, b[1]*8))
        self.assertBEqual(self.mem.read(b[0], b[1]), 0x01020304)
        self.assertBNotEqual(self.mem.read(a[0], a[1]), 0x42)

    def test_overwrite_same(self):
        self.overwrite((0, 4), (0, 4))

    def test_overwrite_simple(self):
        self.overwrite((1, 1), (0, 4))

    def test_overwrite_left(self):
        self.overwrite((0, 1), (0, 4))

    def test_overwrite_right(self):
        self.overwrite((3, 1), (0, 4))

    def test_partial_overwrite_left(self):
        self.mem.write(0, 4, claripy.BVV(0x50515253, 32))
        self.mem.write(2, 4, claripy.BVV(0x60616263, 32))
        self.assertBEqual(self.mem.read(0, 2), 0x5051)
        self.assertBEqual(self.mem.read(2, 4), 0x60616263)
        self.assertBEqual(self.mem.read(0, 6), 0x505160616263)
        self.assertBEqual(self.mem.read(1, 1), 0x51)
        self.assertBEqual(self.mem.read(5, 1), 0x63)

    def test_partial_overwrite_right(self):
        self.mem.write(2, 4, claripy.BVV(0x60616263, 32))
        self.mem.write(0, 4, claripy.BVV(0x50515253, 32))
        self.assertBEqual(self.mem.read(0, 2), 0x5051)
        self.assertBEqual(self.mem.read(2, 4), 0x52536263)
        self.assertBEqual(self.mem.read(0, 6), 0x505152536263)
        self.assertBEqual(self.mem.read(1, 1), 0x51)
        self.assertBEqual(self.mem.read(5, 1), 0x63)

    def test_overwrite_inside(self):
        self.mem.write(10, 6, claripy.BVV(0x112233445566, 8*6))
        self.mem.write(12, 2, claripy.BVV(0x3040, 16))
        self.assertBEqual(self.mem.read(10, 6), 0x112230405566)

    def test_successive_left(self):
        self.mem.write(0, 4, claripy.BVV(0x01020304, 32))
        self.mem.write(4, 4, claripy.BVV(0x05060708, 32))
        self.assertBEqual(self.mem.read(0, 8), 0x0102030405060708)

    def test_successive_right(self):
        self.mem.write(4, 4, claripy.BVV(0x05060708, 32))
        self.mem.write(0, 4, claripy.BVV(0x01020304, 32))
        self.assertBEqual(self.mem.read(0, 8), 0x0102030405060708)

    def test_successive(self):
        self.mem.write(0, 4, claripy.BVV(0x11223344, 32))
        self.mem.write(6, 4, claripy.BVV(0x55667788, 32))
        self.assertBEqual(self.mem.read(0, 11), 0x1122334400005566778800)

    def test_read(self):
        self.mem.write(10, 4, claripy.BVV(0xaabbccdd, 32))
        self.assertBEqual(self.mem.read(8, 4), 0x0000aabb)
        self.assertBEqual(self.mem.read(12, 4), 0xccdd0000)
        self.assertBEqual(self.mem.read(11, 2), 0xbbcc)
        self.assertBEqual(self.mem.read(9, 1), 0)
        self.assertBEqual(self.mem.read(14, 1), 0)

        with self.assertRaises(utils.CodeError):
            self.mem.read(0, 0)
        with self.assertRaises(utils.CodeError):
            self.mem.read(999999999999, 32)

    def test_size(self):
        self.assertEqual(self.mem.size(), 0)
        self.mem.write(10, 4, claripy.BVV(42, 4*8))
        self.assertEqual(self.mem.size(), 14)

    def test_copy(self):
        self.mem.write(1, 1, claripy.BVV(42, 8))
        new_mem = self.mem.copy()
        new_mem.write(1, 1, claripy.BVV(43, 8))
        self.assertBEqual(self.mem.read(1, 1), 42)
        self.assertBEqual(new_mem.read(1, 1), 43)

    def test_hash(self):
        self.mem.write(0, 1, claripy.BVV(42, 8))
        self.mem.write(10, 1, claripy.BVV(43, 8))
        mem_copy = self.mem.copy()
        self.assertEqual(hash(self.mem), hash(mem_copy))

        self.mem.write(5, 1, claripy.BVV(44, 8))
        self.assertNotEqual(hash(self.mem), hash(mem_copy))
        mem_copy.write(5, 1, claripy.BVV(44, 8))
        self.assertEqual(hash(self.mem), hash(mem_copy))

        self.mem.write(0, 10, claripy.BVV(46, 80))
        self.assertNotEqual(hash(self.mem), hash(mem_copy))
        mem_copy.write(0, 10, claripy.BVV(46, 80))
        self.assertEqual(hash(self.mem), hash(mem_copy))

    def test_copy_from(self):
        calldata = CalldataMemory()
        calldata.read(28, 4)
        calldata_24 = calldata.read(24, 4)

        # self.mem[10 to 20] contains calldata[20 to 30]
        self.mem.copy_from(calldata, 10, 20, 10)

        self.assertIs(self.mem.read(14, 4), calldata_24)
        mem_10 = self.mem.read(10, 4)
        self.assertIs(calldata.read(20, 4), mem_10)

        self.assertBEqual(self.mem.read(18, 2), calldata.read(28, 2))

        # Test writing
        self.mem.write(14, 4, claripy.BVV(0x40414243, 32))
        self.assertBEqual(self.mem.read(14, 4), 0x40414243)
        self.assertBEqual(self.mem.read(18, 2), calldata.read(28, 2))


class TestCalldataMemory(unittest.TestCase):
    def setUp(self):
        self.mem = CalldataMemory()

    def test_readonly(self):
        with self.assertRaises(AssertionError):
            self.mem.write(0,1,None)
        with self.assertRaises(AssertionError):
            self.mem.copy_from(0,1,None)

    def test_read_default(self):
        r = self.mem.read(0, 1)
        self.assertTrue(r.symbolic)
